package com.example.tdl.config;


import com.example.tdl.domain.vo.AlarmContactsVo;
import com.example.tdl.domain.vo.ResultWarehouseVo;
import com.example.tdl.domain.vo.WarehouseAlarmLog;
import com.example.tdl.domain.vo.WarehouseCountVo;
import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.WarehouseAlarmLogService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.util.MailUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ScheduleConfig {
    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private WarehouseAlarmLogService warehouseAlarmLogService;

    @Scheduled(cron ="0 0 0/12 * * ?") //每隔一小时执行一次
    public void scheduler() {
        Long endTime  = System.currentTimeMillis();
        Long startTime = endTime-12*60*60*1000l;
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //获取所有的仓库信息
        List<ResultWarehouseVo> resultWarehouseList = warehouseService.getAllWarehouse();
        for(int i =0,length =  resultWarehouseList.size();i<length;i++){
            //获取相关的报警联系人
            List<String> emailList = warehouseService.getEmailByWarehouse(resultWarehouseList.get(i).getWarehouseNo());
            if(emailList.size() == 0){
                continue;
            }
            //查询报警情况
            List<WarehouseCountVo> tempCount =   warehouseAlarmLogService.getCount(resultWarehouseList.get(i).getWarehouseNo(),startTime,endTime,"Temperature");
            List<WarehouseCountVo> humCount =   warehouseAlarmLogService.getCount(resultWarehouseList.get(i).getWarehouseNo(),startTime,endTime,"humity");
            if(tempCount.size() == 0 && humCount.size() == 0){
                continue;
            }
            try {
                MailUtil.sendMail(emailList, "辉度物流监测系统------仓库报警提醒",
                        message(resultWarehouseList.get(i).getWarehouseNo(),sdf.format(new Date(Long.parseLong(String.valueOf(startTime)))),
                                sdf.format(new Date(Long.parseLong(String.valueOf(endTime)))),
                                getSumCount(tempCount),
                                getMaxTime(tempCount),
                                getSumCount(humCount),
                                getMaxTime(humCount)));
            }catch (Exception e){
                e.printStackTrace();
            }

        }
    }


    public String message(String  warehouseNo,String startTime,String endTime,Integer tempCount,String tempTime,Integer humCount,String humTime){
        String html="<!DOCTYPE html>\n" +
                "<html lang=\"en\">\n" +
                "<head>\n" +
                "    <meta charset=\"UTF-8\">\n" +
                "    <title>Title</title>\n" +
                "</head>\n" +
                "<body style=\"align-content: center;\">\n" +
                "    <div style=\"width: 800px;margin: 0px auto;\">\n" +
                "        <div style=\"font-size: 15px;\">\n" +
                "            <p>尊敬的用户，您好：</p>\n" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;自"+startTime+"到"+endTime+",您所管理的仓库编号为"+warehouseNo+"报警情况如下：</p>" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;发生温度报警"+tempCount+"次,主要集中在"+tempTime+";" +
                "            <p>&nbsp;&nbsp;&nbsp;&nbsp;发生湿度报警"+humCount+"次,主要集中在"+humTime+"</p>\n" +
                "            <br>\n" +
                "            <p>致好！</p>\n" +
                "            <br>\n" +
                "        </div>\n" +
                "        <br>\n" +
                "        <div style=\"background-image: url(http://211.161.200.169:38091/test/dist/assets/images/bg.jpg) ; width: 800px;height: 125px;background-repeat: no-repeat;\">\n" +
                "            <div style=\"float: left;\">\n" +
                "                <div style=\"width:240px;height:110px;padding-top:15px;margin-left: 45px;float: left\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitiumGuan.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 13px;font-weight: 300;padding-top: 36px\">辉度官方网站</span>\n" +
                "                    <a href=\"http://www.witium.cn\"><img src=\"http://211.161.200.169:38091/test/dist/assets/images/guan.png\" width=\"110\" height=\"35\"  style=\"margin-top: 3px\"></a>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top:-110px;margin-left: 310px;float: inherit;\" >\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 5px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/WitCloud.png \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 15px;font-weight: 300;padding-top: 50px\">云平台网站</span>\n" +
                "                </div>\n" +
                "                <div style=\"width:240px;height:110px;margin-top: -110px;float: right;\">\n" +
                "                    <div style=\"width:85px; height:85px;float: right;padding-top: 4px;padding-right: 8px\">\n" +
                "                        <img src=\"http://211.161.200.169:38091/test/dist/assets/images/Witium.jpg \" width=\"85\" height=\"85\">\n" +
                "                    </div>\n" +
                "                    <span style=\"display:block;font-size: small;font-family: Arial;margin-left: 35px;font-weight: 300;padding-top: 50px\">辉度微信公众号</span>\n" +
                "                </div>\n" +
                "            </div>\n" +
                "        </div>\n" +
                "        <div style=\"width: 800px;height:40px;font-size: 14px;background-color: #2196F3;\">\n" +
                "            <span style=\"display:block;color: #f8f7ff;text-align: center;padding: 10px;\">\n" +
                "                TEL:086-21-37774020&nbsp;&nbsp;&nbsp;&nbsp;FAX:086-21-37774010&nbsp;&nbsp;&nbsp;&nbsp;Email:sales@witium.com\n" +
                "            </span>\n" +
                "        </div>\n" +
                "    </div>\n" +
                "</body>\n" +
                "</html>";

        return html;
    }


    public Integer getSumCount(List<WarehouseCountVo> counts){
        Integer num = 0;
        for(int i = 0;i<counts.size();i++){
            num = num +counts.get(i).getNum();
        }
        return  num;
    }


    public String  getMaxTime(List<WarehouseCountVo> counts){
        String time="";
        Collections.sort(counts, Comparator.comparing(WarehouseCountVo::getNum));
        time =  counts.get(0).getHours();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date d = format.parse(time);
            Calendar c = Calendar.getInstance();
            c.setTime(d);
            c.add(Calendar.HOUR_OF_DAY, 1);
            String ts = format.format(c.getTime());
            return time + "到" + ts;
        }catch (Exception e){
            e.printStackTrace();
            return  time;
        }
    }
}
