package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.AlarmLog;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/alarmLog")
public class AlarmLogController {
    private Logger logger= LoggerFactory.getLogger(AlarmLogController.class);

    Gson gson= new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private WarehouseAlarmLogService warehouseAlarmLogService;

    private final static String database ="original";

    @Value("${mqtt.userName}")
    private String userName;


    @Value("${mqtt.password}")
    private String password;

    //获取报警信息
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
                "   transportationNo:运输编号" +
                "   description:报警内容" +
                "   startCity:启运城市" +
                "   endCity:目的城市" +
                "   startTime:出发时间（时间戳）" +
                "   endTime:结束时间（时间戳）" +
            "返回值:      transportationNo:运输编号" +
            "   cargoNo:货号" +
            "   startCity:启运城市" +
            "   endCity:目的城市" +
            "   description:报警内容" +
            "   alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getAlarm")
    public Object getAlarm(@RequestBody AlarmTermVo alarmTermVo ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        if(StringUtils.isEmpty(alarmTermVo.getTransportationNo())){
            alarmTermVo.setTransportationNo(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getDescription())){
            alarmTermVo.setDescription(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getEndCity())){
            alarmTermVo.setEndCity(null);
        }
        if(StringUtils.isEmpty(alarmTermVo.getStartCity())){
            alarmTermVo.setStartCity(null);
        }
        if(alarmTermVo.getStartTime()==null){
            alarmTermVo.setStartTime(null);
        }
        if(alarmTermVo.getDescription()==null){
            alarmTermVo.setEndTime(null);
        }
        alarmTermVo.setCompanyNo(user.getCompanyNo());
        return  alarmLogServcie.getAlarm(alarmTermVo);
    }

    //获取最新报警
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  transportationNo:运输编号" +
            "data:报警数据" +
            "description:报警内容" +
            "alarmTime:报警时间" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getRealTimeAlarm")
    public Object getRealTimeAlarm(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo user = gson.fromJson(datum, UserRedisVo.class);
        List<AlarmLogVo> list =   alarmLogServcie.getRealTimeAlarm(user.getCompanyNo());
        return gson.toJson(list);
    }


    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "返回值:  userName:用户名" +
            "password:密码" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @GetMapping("/getAlarmMessage")
    public String getAlarmMessage(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" + token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        return  gson.toJson(new MqttUser(userName,password,userRedisVo.getCompanyNo()));
    }


    //新增报警消息
    @ApiOperation(value = "增加报警",notes = "增加报警")
    @RequestMapping(value = "/addAlarmLog",method = RequestMethod.POST)
    public String addAlarmLog(@RequestBody String addAlarmLog){
        logger.info(addAlarmLog);
        AddAlarmLogVo addAlarmLogVo = gson.fromJson(addAlarmLog,AddAlarmLogVo.class);
        logger.info(System.currentTimeMillis()+"--------"+gson.toJson(addAlarmLogVo));
        if(StringUtils.isEmpty(addAlarmLogVo.getGsn())){
            fb.setCode(0);
            fb.setMessage("网关序列号不能为空");
        }
        if(StringUtils.isEmpty(addAlarmLogVo.getGtype())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
        }
        if(addAlarmLogVo.getAlarm()==null){
            fb.setCode(0);
            fb.setMessage("报警数据不能为空");
        }
        if(addAlarmLogVo.getTs() == null){
            fb.setCode(0);
            fb.setMessage("时间不能为空");
        }
        Map<Integer,String> map = new HashMap<>();
        map.put(0,"正常");
        map.put(1,"等于最大值");
        map.put(2,"不等于最大值");
        map.put(3,"过高");
        map.put(4,"过低");
        map.put(5,"未知");
        if(addAlarmLogVo.getMode() == 1){
            //根据gateway的信息获取线路信息
            String transportationNo =  gatewayService.getTransportationNo(addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype());
            //获取终点，起点
            ResultCircuitVo resultCircuitVo = circuitService.getCircuit(transportationNo);
            for(int i = 0,length=addAlarmLogVo.getAlarm().size();i<length;i++){
                if(addAlarmLogVo.getAlarm().get(i).getAlarmno() !=null && addAlarmLogVo.getAlarm().get(i).getAlarmno() == 0){
                    continue;
                }
                String classify="";
                String desp = "";
                if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("T")){
                    desp = "温度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    classify = "1";
                } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("h")){
                    desp = "湿度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    classify = "2";
                } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("a1")){
                    desp = "震动" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    classify = "3";
                } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("ta")) {
                    desp = "倾角" + map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
                    classify = "4";
                }
                Long  time  = addAlarmLogVo.getAlarm().get(i).getTs();
                if(time.toString().length() == 10){
                    time=time*1000;
                }
                AlarmLog alarmLog = new AlarmLog(transportationNo,resultCircuitVo.getStartCity(),resultCircuitVo.getEndCity(),addAlarmLogVo.getAlarm().get(i).getTdlsn().replace("TDL-","").trim(),desp,addAlarmLogVo.getAlarm().get(i).getValue(),time,classify,addAlarmLogVo.getAlarm().get(i).getDesp(),addAlarmLogVo.getAlarm().get(i).getAlarmno());
                int a = alarmLogServcie.addAlarmLog(alarmLog);
                if(a == 0){
                    fb.setCode(0);
                    fb.setMessage("添加失败");
                }else{
                    fb.setCode(1);
                    fb.setMessage("添加成功");
                }
            }
        }else{
            //根据gateway获取仓库编号
           String warehouseNo =  gatewayService.getWarehouseNo(addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype());
           for(int i = 0,length=addAlarmLogVo.getAlarm().size();i<length;i++){
               if(addAlarmLogVo.getAlarm().get(i).getAlarmno() !=null && addAlarmLogVo.getAlarm().get(i).getAlarmno() == 0){
                   continue;
               }
               String desp = "";
               if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("T")){
                   desp = "温度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
               } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("h")){
                   desp = "湿度" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
               } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("a1")){
                   desp = "震动" +map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
               } else if(addAlarmLogVo.getAlarm().get(i).getAlias().contains("ta")) {
                   desp = "倾角" + map.get(addAlarmLogVo.getAlarm().get(i).getAlarmno());
               }
               WarehouseAlarmLog alarmLog = new  WarehouseAlarmLog(warehouseNo,addAlarmLogVo.getGsn(),addAlarmLogVo.getGtype(),addAlarmLogVo.getAlarm().get(i).getTdlsn().replace("TDL-","").trim(),desp,addAlarmLogVo.getAlarm().get(i).getValue(),addAlarmLogVo.getAlarm().get(i).getTs()*1000,addAlarmLogVo.getAlarm().get(i).getDesp(),addAlarmLogVo.getAlarm().get(i).getAlarmno());
               int a = warehouseAlarmLogService.add(alarmLog);
               if(a == 0){
                   fb.setCode(0);
                   fb.setMessage("添加失败");
               }else{
                   fb.setCode(1);
                   fb.setMessage("添加成功");
               }
           }
        }

        return gson.toJson(fb);
    }


    //获取报警信息
    @ApiOperation(value = "获取报警信息",notes = "获取报警信息" +
            "transportationNo:运输编号" +
            "TDLSN:传感器编号" +
            "time :报警时间（13位时间戳）" +
            "type:地图类型（百度：bd09,谷歌：wgs84）" +
            "返回值：" +
            "transportationNo:线路编号" +
            "temp:温度" +
            "humidity：湿度" +
            "shock：震动" +
            "shockEnergy：震动能量" +
            "titl：倾斜" +
            "lng:经度" +
            "lat：纬度" +
            "time：时间")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getAlarmData")
    public Object getAlarmData(@RequestBody AlarmMessageVo alarmMessageVo,HttpServletRequest request){
        if(StringUtils.isEmpty(alarmMessageVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitService.getNo(alarmMessageVo.getTransportationNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(alarmMessageVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDL"));
            return gson.toJson(fb);
        }
        if(alarmMessageVo.getAlarmTime() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"time"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(alarmMessageVo.getType())){
            alarmMessageVo.setType("wgs84");
        }
        //获取设备的网关信息
        TDLLogVo tdlLogVo =  tdlLogService.getByTransportationNo(alarmMessageVo.getTransportationNo());
        if(tdlLogVo ==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        //获取经纬度
        String sql = "";
        String device = tdlLogVo.getgType()+"_" + tdlLogVo.getgSN();
        CircuitAlarmDataVo circuitAlarmDataVo =new CircuitAlarmDataVo();
        if("bd09".equals(alarmMessageVo.getType())){
            sql = "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\""+device+"\" where time >= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  limit 1;SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\""+device+"\" where time <= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  desc limit 1";
        }else{
            sql =  "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time >= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time  limit 1;SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\""+device+"\" where time <= "+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time desc limit 1;";
        }
        QueryResult queryResult =influxDBTemplate.query(new Query(sql,database));
        if(queryResult.getResults() != null){
            //获取最精确的经纬度
            circuitAlarmDataVo =   parseLocation(queryResult,alarmMessageVo.getAlarmTime());
        }else{
            circuitAlarmDataVo.setLng("");
            circuitAlarmDataVo.setLat("");
        }
        //获取数据
        Map<String,String> map = new HashMap<>();
        map.put("\"T\"","temp");
        map.put("\"h\"","humidity");
        map.put("\"a0\"","shockEnergy");
        map.put("\"a1\"","shock");
        map.put("\"ta\"","tilt");
        String sql1 = "select \"T\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult1 =influxDBTemplate.query(new Query(sql1,database));
        if(queryResult1.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setTemp(parseData(queryResult1,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setTemp("");
        }
        String sql2 = "select \"h\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult2 =influxDBTemplate.query(new Query(sql2,database));
        if(queryResult2.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setHumidity(parseData(queryResult2,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setHumidity("");
        }
        String sql3 = "select \"a0\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult3 =influxDBTemplate.query(new Query(sql3,database));
        if(queryResult3.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setShockEnergy(parseData(queryResult3,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setShockEnergy("");
        }
        String sql4 = "select \"a1\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult4 =influxDBTemplate.query(new Query(sql4,database));
        if(queryResult4.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setShock(parseData(queryResult4,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setShock("");
        }
        String sql5 = "select \"ta\"  FROM \"tdl_policy\".\""+device+"\" where time >="+ alarmMessageVo.getAlarmTime()*1000000l +" ORDER BY time limit 1 ";
        QueryResult queryResult5 =influxDBTemplate.query(new Query(sql5,database));
        if(queryResult5.getResults().get(0).getSeries() != null){
            circuitAlarmDataVo.setTitl(parseData(queryResult5,alarmMessageVo.getAlarmTime()));
        }else{
            circuitAlarmDataVo.setTitl("");
        }
        circuitAlarmDataVo.setTransportationNo(alarmMessageVo.getTransportationNo());
        circuitAlarmDataVo.setTime(alarmMessageVo.getAlarmTime());
        return  gson.toJson(circuitAlarmDataVo);
    }

    //经纬度数据
    public CircuitAlarmDataVo parseLocation(QueryResult queryResult,Long time){
        String t1 ="";
        if(queryResult.getResults().get(0)==null  || queryResult.getResults().get(0) .getSeries() == null){
            t1 = "0";
        }else {
            t1 = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0) == null ? "" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString();
        }
        String t2="";
        if(queryResult.getResults().get(1) == null ||queryResult.getResults().get(1).getSeries()== null){
            t2="0";
        }else{
            t2 = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(0) == null ? "" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(0).toString();
        }
        CircuitAlarmDataVo circuitAlarmDataVo = new CircuitAlarmDataVo();
        try{
            String lng="";
            String lat="";
            if((Long.valueOf(dateToStamp(t1)) +8*60*60*1000-time)<(time-Long.valueOf(dateToStamp(t2))-8*60*60*1000)){
                lng = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null?"" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
                lat = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2) == null?"" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString();
            }else{
                lng = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(1) == null?"" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(1).toString();
                lat = queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(2) == null?"" : queryResult.getResults().get(1).getSeries().get(0).getValues().get(0).get(2).toString();
            }
            circuitAlarmDataVo.setLat(lat);
            circuitAlarmDataVo.setLng(lng);
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
        }
        return  circuitAlarmDataVo;
    }

    //解析数据
    public String parseData(QueryResult queryResult,Long time){
        List<Object> values = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0);
        //判断时间差距
        String t="";
        try{
            t=  dateToStamp(values.get(0).toString());
            logger.info(t);
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            return "";
        }
        if((time -Long.valueOf(t)-60*60*8*1000)>1000*60){
            return "";
        }else{
            //数据
            String value  = values.get(1) == null?"":values.get(1).toString();
            logger.info(value);
            return  value;
        }


    }


    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }
}
