package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.AlarmRedisService;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AlarmRule;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/circuitABB")
public class CircuitABBController {

    private Logger logger = LoggerFactory.getLogger(CircuitABBController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private CircuitABBService circuitABBService;

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private AlarmLogService alarmLogService;

    @Autowired
    private AlarmRedisService alarmRedisService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private ConfigService configService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private I18nController i18n;

    @Autowired
    private TopicService topicService;

    @Autowired
    private TopicConfigService topicConfigService;

    private final static String database ="original";

    //获取本公司的线路信息
    @ApiOperation(value = "获取本公司的线路信息",notes = "获取本公司的线路信息，返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号," +
            "   cargoName:货物别名" +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarmType:预警类型," +
            "   alarm:报警详情（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线,7--在线）," +
            "   transportation:运输方式," +
            "   gSN:网关编号," +
            "   gType:网关类型。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<ResultCircuitForABBVo> list = circuitABBService.getAll(user.getCompanyNo());
        for(int i = 0,length=list.size();i<length;i++){
            if(list.get(i).getCircuitState() == 1){
                //获取设备在线情况
                Object data = infoRedisService.getHash("DeviceOnline",list.get(i).getgType()+"_"+list.get(i).getgSN());
                if(data == null){
                    if(StringUtils.isEmpty(list.get(i).getAlarm())){
                        list.get(i).setAlarm("6");
                    }else{
                        list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                    }
                }else{
                    OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                    if(onlineVo.getOnline() == 0){
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("6");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                        }
                    }else if(onlineVo.getOnline() == 1) {
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("7");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",7");
                        }
                    }
                }
            }else{
                continue;
            }
        }
        return list;
    }

    //根据条件查询
    @ApiOperation(value = "根据条件查询本公司的线路信息",notes = "companyName不用传，根据条件查询:" +
            "   cargoNo:货物编号" +
            "   startTime:出发时间,（时间戳）" +
            "   endTime:目的时间,（时间戳）" +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   time:日期"+
            "返回值说明：" +
            "   transportationNo:运输编号," +
            "   cargoNo:货物编号," +
            "   cargoName:货物别名" +
            "   circuitState:线路状态（0未开始，1运输中，2完成）," +
            "   alarmType:预警类型," +
            "   alarm:报警详情（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线,7--在线）," +
            "   transportation:运输方式," +
            "   gSN:网关编号," +
            "   gType:网关类型。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody CircuitTermVo circuitTermVo,HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(circuitTermVo.getCargoNo())){
            circuitTermVo.setCargoNo(null);
        }
        if(circuitTermVo.getCircuitState() == null || circuitTermVo.getCircuitState() == 3){
            circuitTermVo.setCircuitState(null);
        }
        if(circuitTermVo.getStartTime()==null){
            circuitTermVo.setStartTime(null);
        }
        if(circuitTermVo.getEndTime()==null){
            circuitTermVo.setEndTime(null);
        }
        if(circuitTermVo.getStartTime()!=null && circuitTermVo.getEndTime()!=null  && circuitTermVo.getStartTime()>circuitTermVo.getEndTime()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTime"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(circuitTermVo.getTime())){
            circuitTermVo.setTime(null);
        }else{
            circuitTermVo.setTime(circuitTermVo.getTime().replace("天","").trim());
        }
        circuitTermVo.setCompanyNo(user.getCompanyNo());
        List<ResultCircuitForABBVo> list=circuitABBService.getByTerm(circuitTermVo);
        for(int i = 0,length=list.size();i<length;i++){
            if(list.get(i).getCircuitState() == 1){
                //获取设备在线情况
                Object data = infoRedisService.getHash("DeviceOnline",list.get(i).getgType()+"_"+list.get(i).getgSN());
                if(data == null){
                    if(StringUtils.isEmpty(list.get(i).getAlarm())){
                        list.get(i).setAlarm("6");
                    }else{
                        list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                    }
                }else{
                    OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                    if(onlineVo.getOnline() == 0){
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("6");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",6");
                        }
                    }else if(onlineVo.getOnline() == 1) {
                        if(StringUtils.isEmpty(list.get(i).getAlarm())){
                            list.get(i).setAlarm("7");
                        }else{
                            list.get(i).setAlarm(list.get(i).getAlarm()+",7");
                        }
                    }
                }
            }else{
                continue;
            }
        }
        return list;
    }


    //根据线路编号获取线路详细信息
    @ApiOperation(value = "根据线路编号获取线路详细信息",notes = "根据线路编号获取线路详细信息，传值说明：" +
            "   transportationNo:运输编号；   " +
            "返回值说明：" +
            "   transportationNo：运输编号," +
            "   routeName:线路," +
            "   cargoName:货物别名," +
            "   transportation：运输方式," +
            "   alarmType：预警类型," +
            "   SN:gateway序列号," +
            "   tdlList:{" +
            "       tdlsn：TDLSN序列号" +
            "       batteryVoltage:电池电压}," +
            "   alarmLogList：{" +
            "       transportationNo：运输编号;" +
            "       TDLSN：tag序列号;" +
            "       description：报警描述;" +
            "       data：数值;" +
            "       alarmTime：报警时间;" +
            "       classify：分类}" +
            "   " )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/getDetails")
    public Object getDetails(@RequestBody String transportationNo,HttpServletRequest request) {
        if(StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        //获取线路信息
        CircuitMessage circuitVo = circuitService.getMessage(transportationNo);
        if(circuitVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return  gson.toJson(fb);
        }
        CircuitMessageForABBVo messageVo =  new CircuitMessageForABBVo(circuitVo.getTransportationNo(),circuitVo.getCircuitState(),circuitVo.getRouteName(),circuitVo.getCargoName(),circuitVo.getTransportation(),circuitVo.getAlarmType());
        //获取gateway信息
        TDLLogVo gatewayVo = tdlLogService.getByTransportationNo(transportationNo);
        List<String> TDL= tdlLogService.getTdlSN(transportationNo);
        List<ResultAlarmLog> alarmLogList = new ArrayList<>();
        List<CircuitMessageForABBVo.TDL> tdlList = new ArrayList<>();
        if(gatewayVo == null || TDL.size() == 0){
            messageVo.setSN("NA");
            messageVo.setBattery("NA");
            messageVo.setAlarmLogList(alarmLogList);
            messageVo.setTdlList(tdlList);
            return gson.toJson(messageVo);
        }
        messageVo.setSN(gatewayVo.getgSN());
        alarmLogList= alarmLogService.getByTransportationNo(transportationNo);
        messageVo.setAlarmLogList(alarmLogList);
        String  sql ="";
        if (circuitVo.getEndTime() != null && circuitVo.getEndTime() != 0) {
            sql = "SELECT  \"battery\"  FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where  time >=" + circuitVo.getStartTime() + "  and time<=" + circuitVo.getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
        } else {
            sql = "SELECT \"battery\"   FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where time >=" + circuitVo.getStartTime() + " ORDER BY time desc limit 1";
        }
        QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
        if (queryResult.getResults().get(0).getSeries() == null) {
            messageVo.setBattery("NA");
        }else{
            DecimalFormat df = new DecimalFormat("######0.0");
            String val = queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null ? "" : queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
            Boolean boo = Double.valueOf(val) <0;
            if(Double.valueOf(val) <0){
                messageVo.setBattery("NA");
            }else{
                messageVo.setBattery(df.format(Double.valueOf(val)));
            }
        }
        for(int i=0;i<TDL.size();i++){
            CircuitMessageForABBVo.TDL tdl = new CircuitMessageForABBVo.TDL();
            tdl.setTdlsn(TDL.get(i).replace("TDL-", "").trim());
            if(circuitVo.getStartTime() == null || circuitVo.getStartTime() == 0){
                tdl.setBatteryVoltage("NA");
            }else {
                String  sql1 ="";
                if (circuitVo.getEndTime() != null && circuitVo.getEndTime() != 0) {
                    sql1= "SELECT  \"b\"  FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where \"tdl\"='" + TDL.get(i) + "'  and time >=" + circuitVo.getStartTime() + "  and time<=" + circuitVo.getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
                } else {
                    sql1 = "SELECT \"b\"   FROM \"tdl_policy\".\"" + gatewayVo.getgType() + "_" + gatewayVo.getgSN() + "\" where \"tdl\"='" + TDL.get(i) + "'  and time >=" + circuitVo.getStartTime() + " ORDER BY time desc limit 1";
                }
                QueryResult queryResult1 = influxDBTemplate.query(new Query(sql1, database));
                if (queryResult1.getResults().get(0).getSeries() == null) {
                    tdl.setBatteryVoltage("NA");
                }else{
                    DecimalFormat df = new DecimalFormat("######0.0");
                    String value = queryResult1.getResults().get(0).getSeries().get(0).getValues().get(0).get(1) == null ? "" : queryResult1.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString();
                    if(Double.valueOf(value) <0){
                        tdl.setBatteryVoltage("NA");
                    }else{
                        tdl.setBatteryVoltage(df.format(Double.valueOf(value)));
                    }
                }
            }
            tdlList.add(tdl);
        }
        messageVo.setTdlList(tdlList);
        return gson.toJson(messageVo);
    }

    //添加线路信息ABB
    @ApiOperation(value = "添加线路信息ABB",notes = "添加线路信息ABB，" +
            "   SN:网关编号," +
            "   tdlAlarmList：   " +
            "   {" +
            "       TDLSN:tdl编号," +
            "       alarmType:报警类型," +
            "   }" +
            "   cargoNo:货物编号," +
            "   cargoName:货物别名," +
            "   remark:备注。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addCircuitForABB",method = RequestMethod.POST)
    public Object addCircuitForABB(@RequestBody AddCircuitForABBVo addCircuitForABBVo,HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        ResultCompanyVo companyVo =  companyService.getByCompanyNo(user.getCompanyNo());
        if(StringUtils.isEmpty(addCircuitForABBVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gateway"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addCircuitForABBVo.getCargoNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoNo"));
            return gson.toJson(fb);
        }
        String type =gatewayService.getBySN(addCircuitForABBVo.getSN(),user.getCompanyNo());
        //判断当前gateWay是否存在
        if(StringUtils.isEmpty(type)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayExist"));
            return gson.toJson(fb);
        }
        ResultGatewayVo resultGatewayVo = gatewayService.getBySNAndType(addCircuitForABBVo.getSN(),type);
        if(!resultGatewayVo.getState()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayMounted"));
            return gson.toJson(fb);
        }
        //获取gateway的场景
        if(resultGatewayVo.getUseScene()!= 0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"usedgateway"));
            return gson.toJson(fb);
        }
        String alarmTypeVar="";
        String TDLSNVar="";
        List<String> devList = tdlDeviceService.getByGatewaySN(addCircuitForABBVo.getSN(),type);
        List<Float> tempL = new ArrayList<>();
        List<Float> tempH = new ArrayList<>();
        List<Float> humiL = new ArrayList<>();
        List<Float> humiH = new ArrayList<>();
        List<Integer> pressL = new ArrayList<>();
        List<Integer> pressH = new ArrayList<>();
        List<Integer> sugEnergy = new ArrayList<>();
        List<Float> accThreshold = new ArrayList<>();
        List<Integer> timeThreshold = new ArrayList<>();
        List<Integer> tiltThreshold = new ArrayList<>();
        if (addCircuitForABBVo.getTdlAlarmList().size()==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tdl"));
            return gson.toJson(fb);
        }
        for(int j = 0,length =addCircuitForABBVo.getTdlAlarmList().size();j<length;j++){
            if(StringUtils.isEmpty(addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN())){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"tdl"));
                return gson.toJson(fb);
            }
            //判断当前tdl是否存在
            ResultTDLDeviceVo tdlDeviceVo = tdlDeviceService.getByTDLSN(addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN());
            if(tdlDeviceVo == null){
                fb.setCode(0);
                Object[] args ={addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN()};
                fb.setMessage(i18n.getMessage(request, args,"tdlExist"));
                return gson.toJson(fb);
            }
            if(!StringUtils.isEmpty(tdlDeviceVo.getGatewaySN())){
                fb.setCode(0);
                Object[]  args ={addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN()};
                fb.setMessage(i18n.getMessage(request,args,"tdlUsed"));
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(addCircuitForABBVo.getTdlAlarmList().get(j).getAlarmType())){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"alarmType"));
                return gson.toJson(fb);
            }
            //获取阈值信息
            ResultAlarmVo resultAlarmVo = alarmService.getByType(addCircuitForABBVo.getTdlAlarmList().get(j).getAlarmType(),user.getCompanyNo());
            if (resultAlarmVo==null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"alarmTypeEmpty"));
                return gson.toJson(fb);
            }
            tempL.add(resultAlarmVo.getTemMin());
            tempH.add(resultAlarmVo.getTemMax());
            humiL.add(resultAlarmVo.getHumidityMin());
            humiH.add(resultAlarmVo.getHumidityMax());
            pressL.add(255);
            pressH.add(255);
            sugEnergy.add(1);
            accThreshold.add(Float.valueOf(resultAlarmVo.getShock()));
            timeThreshold.add(5);
            tiltThreshold.add((int)Double.parseDouble(resultAlarmVo.getTilt().toString()));
            alarmTypeVar =alarmTypeVar + addCircuitForABBVo.getTdlAlarmList().get(j).getAlarmType() +"&";
            TDLSNVar =TDLSNVar + addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN() +"&";
            devList.add("TDL-"+addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN());
            AlarmRule.saveAlarmRule("TDL/"+type+"/" +addCircuitForABBVo.getSN()+"/Data",addCircuitForABBVo.getTdlAlarmList().get(j).getTDLSN(),addCircuitForABBVo.getTdlAlarmList().get(j).getAlarmType(),user.getCompanyNo(),alarmService,alarmRedisService);
        }
        //根据sn获取type
        ConfigCMDVo configCMDVo = new ConfigCMDVo("config",5,15,devList,tempL,tempH,humiL,humiH,pressL,pressH,sugEnergy,accThreshold,timeThreshold,tiltThreshold,1,System.currentTimeMillis()/1000l);
        Map<Object,Object> map=new HashMap<>();
        String transportation = "公路";
        String transportationType="本地运输";
        String transportationNo =getTransportNo(transportation,transportationType);
        map.put("transportationNo",transportationNo);
        map.put("cargoNo",addCircuitForABBVo.getCargoNo());
        map.put("cargoName",addCircuitForABBVo.getCargoName());
        map.put("deliveryAddress","站点");
        map.put("transportation",transportation);
        map.put("transportationType",transportationType);
        map.put("createTime",System.currentTimeMillis());
        map.put("remark",addCircuitForABBVo.getRemark());
        map.put("companyNo",user.getCompanyNo());
        map.put("SN",addCircuitForABBVo.getSN());
        map.put("type",type);
        map.put("TDLSNVar",TDLSNVar);
        map.put("alarmTypeVar",alarmTypeVar);
        map.put("TDLCounts",addCircuitForABBVo.getTdlAlarmList()== null ?  0 : addCircuitForABBVo.getTdlAlarmList().size());
        map.put("message",configCMDVo == null ? "" :gson.toJson(configCMDVo));
        map.put("flag",false);
        map.put("untie",false);
        circuitABBService.addCircuitForABB(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加线路成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"circuitSuccess"));
            //增加配置（公司以及线路信息）
            AlarmMqttConfig alarmMqttConfig = new AlarmMqttConfig(transportationNo,companyVo.getCompanyNo());
            infoRedisService.hmSet("AlarmMqttConfig",type+"_"+addCircuitForABBVo.getSN(),gson.toJson(alarmMqttConfig));
            List<ResultTopicVo> topicList = topicService.getByGateway(new GatewaySNAndTypeVo(addCircuitForABBVo.getSN(),type));
            for (ResultTopicVo topic : topicList) {
                infoRedisService.delHashKey("TopicConfig", topic.getTopicName());
                //配置发布还是订阅
                ProbesConfigVo channel_config = new ProbesConfigVo();
                //根据设备的主题名，获取所有的通道
                List<ResultTopicConfigVo> devChannelList = topicConfigService.getProbes(topic.getTopicName(),addCircuitForABBVo.getSN(),type);
                List<ChannelConfigVo> channelConfigs = new ArrayList<>();
                for (ResultTopicConfigVo devChannel : devChannelList) {
                    ChannelConfigVo channelConfig = new ChannelConfigVo(devChannel.getPort(),devChannel.getUnti(),devChannel.getDescription(),devChannel.getAlias());
                    channelConfigs.add(channelConfig);
                }
                channel_config.setIntervalTime(topic.getIntervalTime());
                channel_config.setChannelConfigs(channelConfigs);
                infoRedisService.hmSet("TopicConfig", topic.getTopicName(), channel_config.toString());
            }
        }else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"circuitFailure"));
        }
        return gson.toJson(fb);
    }

    //修改线路
    @ApiOperation(value = "修改线路",notes = "修改线路，传值说明：" +
            "   transportationNo:运输编号，" +
            "   cargoNo:货物编号，" +
            "   cargoName:货物名称。" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateCircuitForABB",method = RequestMethod.POST)
    public Object updateCircuitForABB(@RequestBody UpdateCircuitForABBVo updateCircuitForABBVo,HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateCircuitForABBVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateCircuitForABBVo.getCargoNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"cargoNo"));
            return gson.toJson(fb);
        }
        ResultCircuitForABBVo resultCircuitVo = circuitABBService.getByTransportationNo(updateCircuitForABBVo.getTransportationNo(),user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (resultCircuitVo.getCircuitState()==1){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"inTransit"));
            return gson.toJson(fb);
        }
        int a = circuitABBService.updateCircuitForABB(updateCircuitForABBVo);
        try {
            if (a > 0) {
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"updateCircuitForABBSuccess"));
            } else {
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"updateCircuitForABBFailure"));
            }
        } catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateCircuitForABBFailure"));
        }
        return gson.toJson(fb);
    }

    //删除线路
    @ApiOperation(value = "删除线路",notes = "删除线路，传值说明：" +
            "   transportationNo:运输编号" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delCircuit",method = RequestMethod.POST)
    public Object delCircuit(@RequestBody String transportationNo,HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(transportationNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(transportationNo);
        transportationNo=(String)jsonObject.get("transportationNo");
        ResultCircuitForABBVo resultCircuitVo = circuitABBService.getByTransportationNo(transportationNo,user.getCompanyNo());
        if (resultCircuitVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (resultCircuitVo.getCircuitState()==1){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"inTransit"));
            return gson.toJson(fb);
        }
        TDLLogVo tdlLogVo =  tdlLogService.getByTransportationNo(transportationNo);
        if(tdlLogVo != null &&  resultCircuitVo.getCircuitState() == 0){
            List<String> dev = tdlLogService.getTdlSN(transportationNo);
            if(dev == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            List<String> devList = tdlDeviceService.getByGatewaySN(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(dev == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            String tdl =  tdlDeviceService.getTdl(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(StringUtils.isEmpty(tdl)){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindTDL"));
                return gson.toJson(fb);
            }
            ConfigCMDVo configCMDVo;
            ConfigVo configVo =configService.getConfig(tdlLogVo.getgSN(),tdlLogVo.getgType());
            if(configVo ==null ){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"bindGateway"));
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(configVo.getMessage())){
                configCMDVo = new ConfigCMDVo();
            }else{
                ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
                if(tdl.equals(tdlLogVo.getTDLSN())){
                    configCMDVo = new ConfigCMDVo();
                }else{
                    devList.removeAll(dev);
                    configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
                }
            }
            //解绑gateway和tdl
            Map<Object,Object> map=new HashMap<>();
            map.put("transportationNo",transportationNo);
            map.put("gSN",tdlLogVo.getgSN());
            map.put("gType",tdlLogVo.getgType());
            map.put("endTime",System.currentTimeMillis());
            map.put("message",gson.toJson(configCMDVo));
            circuitService.deleteCircuit(map);
            Map<Object,Object> msg=new HashMap<>();
            msg.put("msg",map.get("msg"));
            if (msg.get("msg").equals("解除网关成功")){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteSuccess"));
                String key ="TDL/"+tdlLogVo.getgType()+"/" +tdlLogVo.getgSN() +"/Data";
                alarmRedisService.delKey(key);
                infoRedisService.delHashKey("AlarmMqttConfig",tdlLogVo.getgType()+"_"+tdlLogVo.getgSN());
            }else{
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
            }
        }else {
            int a = circuitABBService.delCircuit(transportationNo);
            try {
                if (a > 0) {
                    fb.setCode(1);
                    fb.setMessage(i18n.getMessage(request,"circuitDeleteSuccess"));
                } else {
                    fb.setCode(0);
                    fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
                }
            } catch (Exception e){
                e.printStackTrace();
                logger.info(e.toString());
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"circuitDeleteFailure"));
            }
        }
        return gson.toJson(fb);
    }


    //开始运输
    @ApiOperation(value = "开始运输",notes = "开始运输" +
            "startTime:开始时间（13位时间戳）" +
            "transportationNo:运输编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @PostMapping("/startTransportation")
    public  Object  startTransportation(@RequestBody UpdateCircuitStartTimeVo updateCircuitStartTimeVo,HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateCircuitStartTimeVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (circuitABBService.getByTransportationNo(updateCircuitStartTimeVo.getTransportationNo(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (updateCircuitStartTimeVo.getStartTime()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startTime"));
            return gson.toJson(fb);
        }
        //判断该线路是否绑定
        ResultGatewayVo gatewayVo = tdlLogService.getByNo(updateCircuitStartTimeVo.getTransportationNo());
        if(gatewayVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        int a=circuitABBService.startCircuitForABB(updateCircuitStartTimeVo);
        if (a>=0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"startCircuitSuccess"));
        }else {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"startCircuitFailure"));
        }
        return gson.toJson(fb);
    }

    //修改线路结束时间
    @ApiOperation(value = "修改线路结束时间",notes = "修改线路结束时间，传值说明：" +
            "   transportationNo:运输编号," +
            "   endTime:到达时间 " )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateEndTime",method = RequestMethod.POST)
    public Object updateEndTime(@RequestBody UpdateCircuitVo updateCircuitVo,HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateCircuitVo.getTransportationNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"transportationNo"));
            return gson.toJson(fb);
        }
        if (updateCircuitVo.getEndTime()==null){
            updateCircuitVo.setEndTime(System.currentTimeMillis());
        }
        ResultCircuitForABBVo resultCircuitForABBVo=circuitABBService.getByTransportationNo(updateCircuitVo.getTransportationNo(),user.getCompanyNo());
        if (resultCircuitForABBVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"noCircuit"));
            return gson.toJson(fb);
        }
        if (resultCircuitForABBVo.getCircuitState() ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notStarted"));
            return gson.toJson(fb);
        }
        //获取当前线路的gateway 和tdl信息
        TDLLogVo tdlLogVo = tdlLogService.getByTransportationNo(updateCircuitVo.getTransportationNo());
        if(tdlLogVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        List<String> dev = tdlLogService.getTdlSN(updateCircuitVo.getTransportationNo());
        if(dev == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        List<String> devList = tdlDeviceService.getByGatewaySN(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(devList == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        String tdl =  tdlDeviceService.getTdl(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(tdl == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        ConfigCMDVo configCMDVo;
        ConfigVo configVo =configService.getConfig(tdlLogVo.getgSN(),tdlLogVo.getgType());
        if(configVo == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(configVo.getMessage())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"notBound"));
            return gson.toJson(fb);
        }
        ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
        Boolean untie = false;
        if(tdl.equals(tdlLogVo.getTDLSN())){
            configCMDVo =null;
            untie = true;
        }else{
            devList.removeAll(dev);
            configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("transportationNo",updateCircuitVo.getTransportationNo());
        map.put("gSN",tdlLogVo.getgSN());
        map.put("gType",tdlLogVo.getgType());
        map.put("message",configCMDVo == null ? "" : gson.toJson(configCMDVo));
        map.put("endTime",updateCircuitVo.getEndTime());
        map.put("untie",untie);
        circuitABBService.endCircuitForABB(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("修改成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"endCircuitSuccess"));
            String key ="TDL/"+tdlLogVo.getgType()+"/" +tdlLogVo.getgSN() +"/Data";
            alarmRedisService.delKey(key);
            infoRedisService.delHashKey("AlarmMqttConfig",tdlLogVo.getgType()+"_"+tdlLogVo.getgSN());
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"endCircuitFailure"));
        }
        return gson.toJson(fb);
    }


    //生成运输编号
    public String getTransportNo(String transportation,String transportationType){
        String transportationNo="";
        if(transportationType.equals("本地运输")){
            transportationNo = transportationNo+"CHN";
        }else{
            transportationNo = transportationNo+"HUN";
        }
        transportationNo=transportationNo+new SimpleDateFormat("yyyyMMdd").format(new Date());
        if(transportation.equals("公路")){
            transportationNo = transportationNo +"G";
        } else if (transportation.equals("铁路")){
            transportationNo = transportationNo +"T";
        }else if (transportation.equals("空运")){
            transportationNo = transportationNo +"K";
        }else if (transportation.equals("快递")){
            transportationNo = transportationNo +"D";
        }else if (transportation.equals("海运")){
            transportationNo = transportationNo +"H";
        }
        transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
        if (circuitABBService.getNo(transportationNo)>0){
            while (circuitABBService.getNo(transportationNo)>0){
                transportationNo=transportationNo+((int)(Math.random()*(9999-1000+1))+1000);
            }
        }
        return transportationNo;
    }

    //两个时间戳之间相差多少
    public String getMistiming(Long startTime,Long arrivalTime,HttpServletRequest request){
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = startTime- arrivalTime;
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        long sec = diff % nd % nh % nm / ns;
        return day + i18n.getMessage(request,"day")+" " + hour + i18n.getMessage(request,"hour") +" " + min +i18n.getMessage(request,"min") ;
    }


}
