package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.HttpRequester;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;


import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/gateway")
public class GatewayController {

    private Logger logger = LoggerFactory.getLogger(GatewayController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private ModelService modelService;

    @Autowired
    private ProbesService probesService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private TopicService topicService;

    @Autowired
    private TopicConfigService topicConfigService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private I18nController  i18n;

    @Value("${tdl.core.add}")
    private String addmount;

    @Value("${tdl.core.delete}")
    private String delmount;


    //查询所有网关信息
    @ApiOperation(value = "查询所有网关信息",notes = "查询所有网关信息：" +
            "   SN:网关序列号，" +
            "   name:网关名字，" +
            "   type:网关类型，" +
            "   state:网关状态，" +
            "   bConfig:是否配置通道，" +
            "   useScene:使用场景（0为空闲，1为运输，2为仓库）")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return gatewayService.getAll(user.getCompanyNo());
    }

    //添加网关信息
    @ApiOperation(value = "添加网关信息",notes = "添加网关信息，createTime,updateTime不用传，要传的参数：" +
            "   SN:网关序列号，" +
            "   name:网关名字，" +
            "   type:网关类型" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addGateway",method = RequestMethod.POST)
    public Object addGateway(@RequestBody AddGatewayVo addGatewayVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if(!addGatewayVo.getSN().matches("^[0-9]{8}$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySNRule"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayType"));
            return gson.toJson(fb);
        }
        if(addGatewayVo.getType().matches("[u4e00-u9fa5]")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"badGatewayType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addGatewayVo.getModelName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayModelName"));
            return gson.toJson(fb);
        }
        if (modelService.getByModelName(addGatewayVo.getModelName())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existModelName"));
            return gson.toJson(fb);
        }
        if (gatewayService.getBySNAndType(addGatewayVo.getSN(),addGatewayVo.getType())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeatedSN"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",addGatewayVo.getSN());
        map.put("name",addGatewayVo.getName());
        map.put("type",addGatewayVo.getType());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        map.put("companyNo",user.getCompanyNo());
        map.put("modelName",addGatewayVo.getModelName());
        gatewayService.addGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("添加网关成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addGatewaySuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    //修改网关信息
    @ApiOperation(value = "修改网关信息",notes = "修改网关信息，updateTime不用传，要传的参数：" +
            "   name:网关名字，" +
            "   SN:需要修改的网关序列号，" +
            "   type:需要修改的网关类型," +
            "   modelName:需要就传，不需要就不传")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateGateway",method = RequestMethod.POST)
    public Object updateGateway(@RequestBody UpdateGatewayVo updateGatewayVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateGatewayVo.getName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayType"));
            return gson.toJson(fb);
        }
        if (gatewayService.getBySN(updateGatewayVo.getSN(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existGateway"));
            return gson.toJson(fb);
        }
        updateGatewayVo.setUpdateTime(System.currentTimeMillis());
        int a=gatewayService.updateGateway(updateGatewayVo);
        if (a>0){
            if (StringUtils.isEmpty(updateGatewayVo.getModelName())){
                fb.setCode(1);
                fb.setMessage(i18n.getMessage(request,"updateGatewaySuccess"));
            }else {
                Map<Object,Object> map=new HashMap<>();
                map.put("SN",updateGatewayVo.getSN());
                map.put("type",updateGatewayVo.getType());
                map.put("modelName",updateGatewayVo.getModelName());
                gatewayService.deployGateway(map);
                Map<Object,Object> msg=new HashMap<>();
                msg.put("msg",map.get("msg"));
                if (msg.get("msg").equals("通过模板修改设备成功")){
                    fb.setCode(1);
                    fb.setMessage(i18n.getMessage(request,"updateGatewayByModelSuccess"));
                }else{
                    fb.setCode(0);
                    fb.setMessage(msg.get("msg").toString());
                }
            }
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateGatewayFailure"));
        }
        return gson.toJson(fb);
    }

    //删除网关信息
    @ApiOperation(value = "删除网关信息",notes = "删除网关信息，要传的参数：" +
            "   SN:网关序列号，" +
            "   type:网关类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delGateway",method = RequestMethod.POST)
    public Object delGateway(@RequestBody DelGatewayVo delGatewayVo,HttpServletRequest request){
        if (StringUtils.isEmpty(delGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delGatewayVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayType"));
            return gson.toJson(fb);
        }
        ResultGatewayVo  resultGatewayVo = gatewayService.getBySNAndType(delGatewayVo.getSN(),delGatewayVo.getType());
        if (resultGatewayVo==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existGateway"));
            return gson.toJson(fb);
        }
        if(resultGatewayVo.getUseScene()!=0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"beingUsed"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",delGatewayVo.getSN());
        map.put("type",delGatewayVo.getType());
        gatewayService.delGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("删除网关成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteGatewaySuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }

    @ApiOperation(value = "使用通道模板配置通道",notes = "使用通道模板配置通道" +
            "       SN：设备编号" +
            "       type：设备类型" +
            "       modelName：模板名")
    @RequestMapping(value = "/deployProbes" ,method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    public String deployProbes(@RequestBody DeployProbesVo deployProbesVo,HttpServletRequest request){
        if(StringUtils.isEmpty(deployProbesVo.getModelName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayModelName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(deployProbesVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(deployProbesVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayType"));
            return gson.toJson(fb);
        }
        if(gatewayService.getBySNAndType(deployProbesVo.getSN(),deployProbesVo.getType())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existGateway"));
            return gson.toJson(fb);
        }
        //根据SN,type获取设备状态
        ResultGatewayVo  resultGatewayVo = gatewayService.getBySNAndType(deployProbesVo.getSN(),deployProbesVo.getType());
        if(resultGatewayVo.getState()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"isState"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("SN",deployProbesVo.getSN());
        map.put("type",deployProbesVo.getType());
        map.put("modelName",deployProbesVo.getModelName());
        gatewayService.deployGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("通过模板修改设备成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }


    //挂载设备
    @ApiOperation(value = "设备挂载/xie",notes = "使用通道模板配置通道" +
            "       SN：设备编号" +
            "       type：设备类型" +
            "       state：挂载-0/卸载-1" +
            "       bConfig:是否配置通道")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/setState",method = RequestMethod.POST)
    public String setState(@RequestBody GatewayState device){
        logger.info("t1:"   + System.currentTimeMillis());
        if (device.getbConfig() == false) {
            fb.setCode(0);
            fb.setMessage("该网关未配置通道");
            return gson.toJson(fb);
        }
        //根据设备的SN,TYPE获取所有通道
        List<ResultProbesVo> list = probesService.getProbesByGateway(device.getSN(),device.getType());
        logger.info("t2:"   + System.currentTimeMillis());
        if (list.size() == 0) {
            fb.setCode(0);
            fb.setMessage("该网关未配置通道");
            return gson.toJson(fb);
        }
        DeviceConfigVo deviceConfig = new DeviceConfigVo();
        //获取网关创建时间
        ResultGatewayVo resultGatewayVo = gatewayService.getBySNAndType(device.getSN(),device.getType());
        logger.info("t3:"   + System.currentTimeMillis());
        Integer New = 0;
        if(resultGatewayVo.getCreateTime()>1535940000000l){
            deviceConfig.setType(1);
            New =1;
        }else{
            deviceConfig.setType(0);
        }
        List<GConfigVo> configList = new ArrayList<>();
        for (ResultProbesVo devChannel : list) {
            GConfigVo con = new GConfigVo(devChannel.getPort(), devChannel.getUnit(), devChannel.getDescription(), devChannel.getAlias());
            configList.add(con);
        }
        deviceConfig.setConfigList(configList);
        infoRedisService.hmSet("DeviceConfig", device.getType() + "_" + device.getSN(), deviceConfig.toString());
        //查询该设备所有的主题类型0-发布，1-订阅，2-发布并订阅
        List<ResultTopicVo> topicList = topicService.getByGateway(new GatewaySNAndTypeVo(device.getSN(),device.getType()));
        logger.info("t4:"   + System.currentTimeMillis());
        //将数据放到redis中
        List<String> pubList = new ArrayList<>();
        List<String> subList = new ArrayList<>();
        for (ResultTopicVo topic : topicList) {
            //配置发布还是订阅
            if (topic.getType() == 0) {
                pubList.add(gson.toJson(topic.getTopicName()));
            } else if (topic.getType() == 1) {
                subList.add(gson.toJson(topic.getTopicName()));
            } else if (topic.getType() == 2) {
                pubList.add(gson.toJson(topic.getTopicName()));
                subList.add(gson.toJson(topic.getTopicName()));
            }
            ProbesConfigVo channel_config = new ProbesConfigVo();
            //根据设备的主题名，获取所有的通道
            List<ResultTopicConfigVo> devChannelList = topicConfigService.getProbes(topic.getTopicName(),device.getSN(),device.getType());
            List<ChannelConfigVo> channelConfigs = new ArrayList<>();

            for (ResultTopicConfigVo devChannel : devChannelList) {
                ChannelConfigVo channelConfig = new ChannelConfigVo(devChannel.getPort(),devChannel.getUnti(),devChannel.getDescription(),devChannel.getAlias());
                channelConfigs.add(channelConfig);
            }
            channel_config.setIntervalTime(topic.getIntervalTime());
            channel_config.setChannelConfigs(channelConfigs);
            if (device.getState() == 0) {
                infoRedisService.hmSet("TopicConfig", topic.getTopicName(), channel_config.toString());
            } else {
                infoRedisService.delHashKey("TopicConfig", topic.getTopicName());
            }
        }
        logger.info("t5:"   + System.currentTimeMillis());
        infoRedisService.hmSet("SubTopic", device.getType() + "_" + device.getSN(), subList.toString());
        infoRedisService.hmSet("PubTopic", device.getType() + "_" + device.getSN(), pubList.toString());
        if (device.getState() == 0) {
            //挂载
            logger.info("t6:"   + System.currentTimeMillis());
            String sr = HttpRequester.sendPost(addmount, gson.toJson(new GatewaySNAndTypeVo(device.getSN(), device.getType(),New)));
            logger.info("t7:"   + System.currentTimeMillis());
            device.setState(1);
        } else if (device.getState() == 1) {
            //卸载
            logger.info("t6:"   + System.currentTimeMillis());
            String sr = HttpRequester.sendPost(delmount, gson.toJson(new GatewaySNAndTypeVo(device.getSN(), device.getType())));
            logger.info("t7:"   + System.currentTimeMillis());
            device.setState(0);
            infoRedisService.delHashKey("DeviceConfig", device.getType() + "_" + device.getSN());
            infoRedisService.delHashKey("SubTopic", device.getType() + "_" + device.getSN());
            infoRedisService.delHashKey("PubTopic", device.getType() + "_" + device.getSN());
        }
        try{
            logger.info("t8:"   + System.currentTimeMillis());
            Boolean boo = gatewayService.updateState(device);
            logger.info("t9:"   + System.currentTimeMillis());
            if (!boo) {
                fb.setCode(0);
                fb.setMessage("修改失败");
            } else {
                fb.setCode(1);
                fb.setMessage("修改成功");
            }
        } catch (Exception e) {
            logger.info("t10:"   + System.currentTimeMillis());
            e.printStackTrace();
            logger.info(e.toString());
        }
        return gson.toJson(fb);
    }

    //获取未使用的网关
    @ApiOperation(value = "获取未使用的网关",notes = "获取未使用的网关，传值说明：" +
            "   gSN:需要查询的网关序列号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/getUnusedGateway",method = RequestMethod.POST)
    public Object getUnusedGateway(@RequestBody String gSN, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(gSN)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gateway"));
            return  gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(gSN);
        gSN=(String)jsonObject.get("gSN");
        return gatewayService.getUnusedGateway(gSN,user.getCompanyNo());
    }

    //分配网关
    @ApiOperation(value = "分配网关",notes = "分配网关，传值说明：" +
            "   SN:需要分配的网关序列号，" +
            "   type:需要分配的网关类型，" +
            "   companyNo:分配的公司编号。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/updateCompany",method = RequestMethod.POST)
    public Object updateCompany(@RequestBody UpdateGatewayCompanyVo updateGatewayCompanyVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(updateGatewayCompanyVo.getCompanyNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"companyNo"));
            return  gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateGatewayCompanyVo.getSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return  gson.toJson(fb);
        }
        if(StringUtils.isEmpty(updateGatewayCompanyVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewayType"));
            return  gson.toJson(fb);
        }
        int a=gatewayService.updateCompany(updateGatewayCompanyVo);
        if (a>=0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"updateGatewayCompanySuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"updateGatewayCompanyFailure"));
        }
        return gson.toJson(fb);
    }


    //获取设备类型
    @ApiOperation(value = "分配网关",notes = "分配网关，传值说明：" +
            "   SN:需要分配的网关序列号，" +
            "   type:需要分配的网关类型，" +
            "   companyNo:分配的公司编号。")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/getGatewayType",method = RequestMethod.GET)
    public Object getGatewayType(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return   gatewayService.getGatewayType(user.getCompanyNo());
    }

}
