package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;

import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/homePage")
public class HomePageController {
    private Logger logger = LoggerFactory.getLogger(HomePageController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private CircuitABBService circuitABBService;

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private WarehouseService warehouseService;

    private final static String database ="original";

    @Autowired
    private I18nController  i18n;

    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "sum:总数量;" +
            "onRoute:在途数量;" +
            "delay:延误数量;" +
            "alarmed: 报警数量;" +
            "bump：颠簸数量;" +
            "list: " +
            "   {" +
            "       classify:报警描述;"+
            "       count:数量;" +
            "   }"+
            "alarmLogVoList :"+
            "   {" +
            "       description:报警描述;" +
            "       data:数值;" +
            "       alarmTime:报警时间;" +
            "   }" +
            "circuitList：" +
            "   {" +
            "       transportationNo:运输编号;" +
            "       cargoNo：货物编号;" +
            "       startCity:开始城市;" +
            "       endCity:目的城市;" +
            "       startTime:开始时间;" +
            "       endTime:到达时间;" +
            "       lng:当前经度;" +
            "       lat:当前纬度;" +
            "       time:当前时间;" +
            "   }" +
            "histogramDataList：" +
            "   {" +
            "       classify: 种类（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线）" +
            "       count：数量" +
            "   }")
    @PostMapping("/getAll")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getAll(@RequestBody String time, HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        time=(String) JSON.parseObject(time).get("time");
        time = time.replace("天","").trim();

        HomePageDataVo dataVo = new HomePageDataVo();
        //总数量
        dataVo.setSum(circuitService.getCount(time,System.currentTimeMillis()/1000l,user.getCompanyNo()));
        //在途数量
        dataVo.setOnRoute(circuitService.getOnRoute(user.getCompanyNo()));
        Integer delay =  circuitService.getDelayCount(System.currentTimeMillis(),time,user.getCompanyNo());
        //报警数量
        dataVo.setAlarmed( alarmLogServcie.getCount(time,user.getCompanyNo()));
        //获取颠簸数量
        dataVo.setBump(alarmLogServcie.getBumpCount(time,user.getCompanyNo()));
//        dataVo.setAlarmLogVoList(alarmLogServcie.getAlarmLog(time,user.getCompanyNo()));//报警列表
        dataVo.setWarehouseVoList(warehouseService.getAll(user.getCompanyNo()));
        //获取报警柱状图
        List<HistogramData> list = alarmLogServcie.getByCount(time,user.getCompanyNo());
        list.add(new HistogramData("5",delay));
        //获取所有的仓库数据
        for(int j = 0,length = list.size();j<length;j++){
            if(list.get(j).getClassify().equals("5")){
                Integer count = list.get(j).getCount() +delay;
                list.get(j).setCount(count);
                dataVo.setDelay(list.get(j).getCount());//延误数量
            }
        }
        //获取设备的当前位置
        List<ResultCircuitVo> circuitVos =  circuitService.getByTime(user.getCompanyNo());
        List<CircuitVo> circuitList = new ArrayList<>();
        Integer num = 0;
        for(int i = 0;i<circuitVos.size();i++){
            CircuitVo circuitVo = new CircuitVo();
            circuitVo.setTransportationNo(circuitVos.get(i).getTransportationNo());
            circuitVo.setCargoNo(circuitVos.get(i).getCargoNo());
            circuitVo.setStartCity(circuitVos.get(i).getStartCity());
            circuitVo.setEndCity(circuitVos.get(i).getEndCity());
            circuitVo.setStartTime(circuitVos.get(i).getStartTime());
            circuitVo.setEndTime(circuitVos.get(i).getEndTime());
            ResultGatewayVo gatewayVo  = tdlLogService.getByNo(circuitVos.get(i).getTransportationNo());
            String device = gatewayVo.getType() + "_" + gatewayVo.getSN();
            Object data = infoRedisService.getHash("DeviceOnline",   device );
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 0){
                    num =num +1;
                }
            }
            if (circuitVos.get(i).getStartTime() == null || circuitVos.get(i).getStartTime() == 0) {
                circuitList.add(circuitVo);
            } else {
                if(gatewayVo==null){
                    circuitList.add(circuitVo);
                }else {
                    Long startTime = circuitVos.get(i).getStartTime() * 1000000l;
                    String sql =  "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"  where   time >= " + startTime + " ORDER BY time  desc limit 1";
                    QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
                    if (queryResult.getResults().get(0).getSeries() != null) {
                        circuitVo.setTime(parseTime(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                        circuitVo.setLng(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                        circuitVo.setLat(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    }
                    circuitList.add(circuitVo);
                }
            }
        }
        list.add(new HistogramData("6",num));
        dataVo.setHistogramDataList(list);
        dataVo.setCircuitList(circuitList);
        return gson.toJson(dataVo);
    }

    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "   sum:总数量;" +
            "   onRoute:在途数量;" +
            "   alarmed: 报警数量;" +
            "   bump：颠簸数量;" +
            "   alarmLogABBVos : {" +
            "       transportationNo:线路编号;" +
            "       description:报警描述;" +
            "       data:数值;" +
            "       alarmTime:报警时间;" +
            "   }" +
            "   circuitABBVos：{" +
            "       transportationNo:运输编号;" +
            "       gatewaySN：网关编号;" +
            "       TDLSN：传感器编号;" +
            "       cargoNo：货物编号;" +
            "       cargoName：货物名;" +
            "       lng:当前经度;" +
            "       lat:当前纬度;" +
            "       isAlarm:该线路是否发生过报警;" +
            "       time:当前时间;" +
            "   }" +
            "   warehouseVoList：{" +
            "       warehouseNo:仓库编号;" +
            "       img：仓库图片;" +
            "       warehouseName：仓库名;" +
            "       country：国家;" +
            "       city：城市;" +
            "       regions：区域;" +
            "       addressDetail:地址;" +
            "       regionName:区域名;" +
            "       lng:经度;" +
            "       lat:纬度;" +
            "       property:属性;" +
            "       type:类型;" +
            "       hardwareLevel:硬件等级;" +
            "       areaIndoor:室内平面面积;" +
            "       areaRack:货架面积;" +
            "       areaOutdoor:室外面积;" +
            "       areaPlanIndoor:实际室内面积;" +
            "       office:办事处;" +
            "       timeZone:时区;" +
            "       warehouseInfo:基础描述;" +
            "       companyNo:公司编号;" +
            "       companyName:公司名;" +
            "       remark:当前时间;" +
            "   }" +
            "   histogramDataList{" +
            "       classify: 种类（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线）" +
            "       count：数量" +
            "   }")
    @PostMapping("/getAllForABB")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getAllForABB(@RequestBody String time, HttpServletRequest request){
        String datum = tokenRedisService.get("TOKEN_" +request.getHeader("Account_token"));
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        time=(String) JSON.parseObject(time).get("time");
        time = time.replace("天","").trim();
        //获取数量
        HomePageDataABBVo dataVo = new HomePageDataABBVo();
        //获取所有的线路数量
        dataVo.setSum(circuitABBService.getCount(time,System.currentTimeMillis()/1000l,user.getCompanyNo()));//总数量
        //获取近几天的线路信息
        dataVo.setOnRoute(circuitABBService.getOnRoute(user.getCompanyNo()));//在途数量
        dataVo.setAlarmed(alarmLogServcie.getCount(time,user.getCompanyNo()));//报警数量
        dataVo.setBump(alarmLogServcie.getBumpCount(time,user.getCompanyNo())); //获取颠簸数量
//        dataVo.setAlarmLogVos(alarmLogServcie.getAlarmLog(time,user.getCompanyNo()));//报警列表
        dataVo.setWarehouseVoList(warehouseService.getAll(user.getCompanyNo()));

        //获取设备的当前位置
        List<CircuitABBVo> circuitVos = circuitABBService.getForHomePage(user.getCompanyNo());
        Integer num = 0;
        for(int i = 0;i<circuitVos.size();i++){
            ResultGatewayVo gatewayVo = tdlLogService.getByNo(circuitVos.get(i).getTransportationNo());
            String device = gatewayVo.getType() + "_" + gatewayVo.getSN();
            Object data = infoRedisService.getHash("DeviceOnline",device);
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 0){
                    num =num +1;
                }
            }
            if(gatewayVo!=null&&circuitVos.get(i).getBindTime()!=null){
                Long startTime = circuitVos.get(i).getBindTime() * 1000000l;
                String sql =  "SELECT \"bdlng\",\"bdlat\" FROM \"tdl_policy\".\"" + device + "\" where  \"data_type\" = 'location' and  time >= " + startTime + " ORDER BY time desc limit 1";
                QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
                if (queryResult.getResults().get(0).getSeries() != null) {
//                    circuitVos.get(i).setTime(parseTime(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                    circuitVos.get(i).setLng(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                    circuitVos.get(i).setLat(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                }
            }else {
//                circuitVos.get(i).setTime(null);
                circuitVos.get(i).setLng(null);
                circuitVos.get(i).setLat(null);
            }
        }
        List<HistogramData> list = alarmLogServcie.getByCountForABB(time,user.getCompanyNo());//获取报警柱状图
        list.add(new HistogramData("6",num));
        dataVo.setHistogramDataList(list);
        dataVo.setCircuitABBVos(circuitVos);
        return gson.toJson(dataVo);
    }

    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "onRoute:在途数量;" +
            "delay:延误数量;" +
            "alarmed: 报警数量;")
    @PostMapping("/getAllForAPP")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getAllForAPP(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        AlarmDataVo alarmDataVo = new AlarmDataVo();
        alarmDataVo.setOnRoute(circuitService.getOnRoute(user.getCompanyNo()));
        alarmDataVo.setAlarmed(alarmLogServcie.getAlarmCount(user.getCompanyNo()));
        alarmDataVo.setDelay(circuitService.getDelay(System.currentTimeMillis(),user.getCompanyNo()));
        return gson.toJson(alarmDataVo);
    }

    //解析时间
    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }
}
