package com.example.tdl.web;

import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.LoginUserVo;
import com.example.tdl.domain.vo.UserRedisVo;
import com.example.tdl.domain.vo.UserVo;
import com.example.tdl.entity.LoginLog;
import com.example.tdl.service.LoginLogService;
import com.example.tdl.service.UserService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.service.redis.UserTokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@RestController
@RequestMapping("/login")
public class LoginController {

    private Logger logger = LoggerFactory.getLogger(LoginController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private UserService userService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private UserTokenRedisService userTokenService;

    @Autowired
    private LoginLogService loginLogService;

    @Autowired
    private I18nController i18n;

    //使用token登录
    @ApiOperation(value = "使用token登录",notes = "使用token登录:" +
            "      userName: 用户名," +
            "      password: 密码")
    @RequestMapping(value = "/loginForToken", method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = true, dataType = "String"),
    })
    public String loginForToken(@RequestBody LoginUserVo loginUserVo, HttpServletRequest request, HttpServletResponse response) {
        if(StringUtils.isEmpty(loginUserVo.getUserName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userName"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(loginUserVo.getPassword())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"password"));
            return gson.toJson(fb);
        }
        try {
            UserVo userVo=userService.getByUserName(loginUserVo.getUserName());
            //将封装好的用户名和密码交给shiro安全框架并实现登陆
            SecurityUtils.getSubject().login(new UsernamePasswordToken(loginUserVo.getUserName(), loginUserVo.getPassword()));
            //生成token
            String token = DigestUtils.md5Hex(System.currentTimeMillis() + userVo.getUserName());
            UserRedisVo redisVo = new UserRedisVo(userVo.getUserName(),userVo.getUserNumber(),userVo.getNickName(),userVo.getEmail(),userVo.getPhone(),userVo.getRoleName(),userVo.getCompanyName(),userVo.getCompanyNo());
            //将用户数据保存到redis中
            userTokenService.setHash(userVo.getUserNumber(),"TOKEN_" + token,System.currentTimeMillis()+"",60*60*24);
            tokenRedisService.set("TOKEN_" + token, gson.toJson(redisVo), 60 * 30);
            String ip = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest().getRemoteAddr();
            int a=loginLogService.addLoginLog(new LoginLog(loginUserVo.getUserName(),System.currentTimeMillis(),ip));
            if (a>0){
                fb.setCode(1);
                fb.setMessage(token);
            }else{
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"addLogFailure"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"login"));
        }
        return gson.toJson(fb);
    }

    //注销
    @ApiOperation(value = "注销",notes = "注销登录")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/logout", method = RequestMethod.GET)
    public String logout(HttpServletRequest request){
        String token=request.getHeader("Account_token");
        tokenRedisService.delKey("TOKEN_"+token);
        fb.setCode(1);
        fb.setMessage("注销成功");
        return gson.toJson(fb);
    }


//    @ApiOperation(value="登录",notes = "登录")
//    @RequestMapping(value = "/login", method = RequestMethod.POST)
//    public String login(@RequestBody LoginUserVo loginUserVo,HttpServletRequest request,HttpServletResponse response){
//        if(StringUtils.isEmpty(loginUserVo.getUserName())){
//            fb.setCode(0);
//            fb.setMessage("用户名不能为空");
//            return gson.toJson(fb);
//        }
//        if(StringUtils.isEmpty(loginUserVo.getPassword())){
//            fb.setCode(0);
//            fb.setMessage("密码不能为空");
//            return gson.toJson(fb);
//        }
//        String token = userService.getUser(loginUserVo);
//        if(token == null){
//            fb.setCode(0);
//            fb.setMessage("账号或者密码错误");
//        }else{
//            CookieUtils.setCookie(request,response,"Account_token",token);
//            fb.setCode(1);
//            fb.setMessage(token);
//        }
//        return gson.toJson(fb);
//    }
//
//

}
