package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.entity.Alarm;
import com.example.tdl.entity.Gateway;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.expression.spel.ast.NullLiteral;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/warehouse")
public class WarehouseController {

    private Logger logger = LoggerFactory.getLogger(WarehouseController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private WarehouseService warehouseService=new WarehouseService();

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private I18nController i18n;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private UserService userService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private TDLLogService tdlLogService;

    private final static String database ="original";



    //查询所有仓库信息
    @ApiOperation(value = "查询所有仓库信息",notes = "查询所有仓库信息,返回值说明：" +
            "   warehouseName:仓库名," +
            "   warehouseNo:仓库编号," +
            "   img:图片路径" +
            "   country:国家," +
            "   city:市," +
            "   regions:国家地区," +
            "   addressDetail:具体地址," +
            "   regionName:区域," +
            "   lng:经度," +
            "   lat:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   companyName:所属公司," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        return warehouseService.getAll(userRedisVo.getCompanyNo());
    }



    //根据条件查询仓库信息
    @ApiOperation(value = "根据条件查询仓库信息",notes = "根据条件查询仓库信息,传值说明：" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   city:市," +
            "   property:属性," +
            "   type:,仓库类型" +
            "   hardwareLevel:硬件等级。(查询全部传{})" +
            "返回值说明：" +
            "   warehouseName:仓库名," +
            "   warehouseNo:仓库编号," +
            "   country:国家," +
            "   city:市," +
            "   regions:国家地区," +
            "   addressDetail:具体地址," +
            "   regionName:区域," +
            "   lng:经度," +
            "   lat:纬度," +
            "   property:属性," +
            "   type:类型," +
            "   alarmType:预警类型," +
            "   alarmScene:预警场景," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   companyName:公司," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping(value="/getWarehouseByTerm")
    public Object getWarehouseByTerm(@RequestBody WarehouseTermVo warehouseTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(warehouseTermVo.getCity())){
            warehouseTermVo.setCity(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getCountry())){
            warehouseTermVo.setCountry(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getHardwareLevel())){
            warehouseTermVo.setHardwareLevel(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getProperty())){
            warehouseTermVo.setProperty(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getType())){
            warehouseTermVo.setType(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getWarehouseName())){
            warehouseTermVo.setWarehouseName(null);
        }
        warehouseTermVo.setCompanyNo(userRedisVo.getCompanyNo());
        return warehouseService.getAllByTerm(warehouseTermVo);
    }



    //获取仓库以及设备数据
    @ApiOperation(value = "获取仓库以及设备数据",notes = "获取仓库以及设备数据,传值说明：" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   city:市," +
            "   alarm:预警(查询全部传{})" +
            "返回值:"+
            "   warehouseName:仓库名," +
            "   warehouseNo:仓库编号," +
            "   regions:国家地区," +
            "   counts:设备数量," +
            "   temp:当前温度," +
            "   humidity:当前温度" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping(value="/getByTerm")
    public Object getByTerm(@RequestBody WareHouseByTermVo warehouseTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        warehouseTermVo.setCompanyNo(userRedisVo.getCompanyNo());
        if(StringUtils.isEmpty(warehouseTermVo.getCity())){
            warehouseTermVo.setCity(null);
        }
        if(StringUtils.isEmpty(warehouseTermVo.getCountry())){
            warehouseTermVo.setCountry(null);
        }
        List<SimpleWarehouseVo>  simpleWarehouseVos = warehouseService.getByWarehouse(warehouseTermVo);
        for( int i = 0,length= simpleWarehouseVos.size();i<length;i++ ){
            //判断是否已经绑定设备
            if(simpleWarehouseVos.get(i).getCount() == 0){
                simpleWarehouseVos.get(i).setTemp("~");
                simpleWarehouseVos.get(i).setHumidity("~");
                continue;
            }
            List<DeviceVo>  list = warehouseService.getDevice(simpleWarehouseVos.get(i).getWarehouseNo());
            if(list.size() == 0){
                simpleWarehouseVos.get(i).setTemp("~");
                simpleWarehouseVos.get(i).setHumidity("~");
                continue;
            }
            String  sql = "";
            for(int j = 0,len=list.size();j<len;j++){
                if( StringUtils.isEmpty(list.get(j).getTdl())){
                    simpleWarehouseVos.get(i).setTemp("~");
                    simpleWarehouseVos.get(i).setHumidity("~");
                    continue;
                }
                String tdl = list.get(j).getTdl().replace(",","\' or \"tdl\" = \'");
//                sql = "SELECT \"T\",\"h\" FROM \"tdl_policy\".\"WTD93LG_17110194\"  where  \"tdl\" = \'TDL-1706190193608\' or \"tdl\" = \'TDL-1701110082608\' GROUP BY \"tdl\"  order by time  desc limit 1";
                sql = "SELECT \"T\",\"h\" FROM \"tdl_policy\".\""+list.get(j).getDevice()+"\"  where  \"tdl\" = \'"+ tdl  +"\'  and  time >="+ list.get(j).getStartTime() +"   GROUP BY \"tdl\"  order by time  desc limit 1";
            }
            if(StringUtils.isEmpty(sql)){
                simpleWarehouseVos.get(i).setTemp("~");
                simpleWarehouseVos.get(i).setHumidity("~");
                continue;
            }
            QueryResult queryResult  = influxDBTemplate.query(new Query(sql,database));
            if(queryResult.getResults().get(0).getSeries()== null){
                simpleWarehouseVos.get(i).setTemp("~");
                simpleWarehouseVos.get(i).setHumidity("~");
                continue;
            }
            //解析温湿度数据
            SimpleWarehouseVo simpleWarehouseVo =   dataData(queryResult);
            simpleWarehouseVos.get(i).setHumidity(simpleWarehouseVo.getHumidity());
            simpleWarehouseVos.get(i).setTemp(simpleWarehouseVo.getTemp());

        }
        return  simpleWarehouseVos;
    }



    //根据仓库编号获取仓库信息
    @ApiOperation(value = "根据仓库编号获取仓库信息",notes = "根据仓库编号获取仓库信息,传值说明：" +
            "   {" +
            "       warehouseNo: 仓库编号," +
            "       warehouseName: 仓库名," +
            "       country: 国家," +
            "       city: 城市," +
            "       addressDetail: 详细地址," +
            "       gatewayCounts: 仓库下网关数量," +
            "       tdlCounts: 仓库下TDL数量," +
            "       property: 仓库属性," +
            "       type: 仓库类型," +
            "       hardwareLevel: 硬件等级," +
            "       areaPlanIndoor: 实际室内总面积," +
            "       gatewayOnLineVos: [" +
            "           {" +
            "               type: 网关类型," +
            "               state: 在线状态（6离线，7在线）," +
            "               battery: 电量," +
            "               tdlDeviceDetailVos: [" +
            "                   {" +
            "                       pressure: 电压," +
            "                       temperature: 温度," +
            "                       humidity: 湿度," +
            "                       time: 时间," +
            "                       tdlsn: TDL编号" +
            "                       gatewaySN:网关编号" +
            "                   }" +
            "               ]," +
            "               sn: 设备编号" +
            "           }" +
            "       ]" +
            "   }")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getDetails")
    public Object getDetails(@RequestBody String warehouseNo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo userRedisVo = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(warehouseNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        if(warehouseService.getWarehouseNo(warehouseNo,userRedisVo.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        WarehouseDetailVo warehouseDetailVo=warehouseService.getWarehouseDetailByNo(warehouseNo);
        //获取
        List<WarehouseLogVo>  list =tdlLogService.getByWarehouse(warehouseNo);
        List<GatewayOnLineVo> gatewayOnLineVos=new ArrayList<>();
        for (int a=0;a<list.size();a++){

            if(list.get(a) == null){
                continue;
            }
            if(list.get(a).getEndTime()!=null){
                continue;
            }
            GatewayOnLineVo gatewayOnLineVo = new GatewayOnLineVo();
            gatewayOnLineVo.setgSN(list.get(a).getSN());
            gatewayOnLineVo.setType(list.get(a).getType());
            Object data = infoRedisService.getHash("DeviceOnline", list.get(a).getDevice() );
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 1){
                    gatewayOnLineVo.setState(7);

                }else {
                    gatewayOnLineVo.setState(6);
                }
            }else{
                gatewayOnLineVo.setState(6);
            }
            //根据网关线路编号获取时间
            Long startTime =  tdlLogService.getTimeByWarehouseNo(warehouseNo,list.get(a).getSN());
            String sql ="SELECT \"battery\" FROM \"tdl_policy\".\""+list.get(a).getDevice() +"\"where time >="+startTime*1000000l+" ORDER BY time DESC LIMIT 1";
            QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
            if (queryResult.getResults().get(0).getSeries() != null) {
                if(Double.valueOf(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString())>0){
                    gatewayOnLineVo.setBattery(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1)==null ? "NA" :queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                }else{
                    gatewayOnLineVo.setBattery("NA");
                }
            }else {
                gatewayOnLineVo.setBattery("NA");
            }
            List<String> tdls=tdlDeviceService.getByGatewaySN(list.get(a).getSN(),list.get(a).getType());
            List<TDLDeviceDetailVo> tdlDeviceDetailVos=new ArrayList<>();
            for (int b=0;b<tdls.size();b++){
                TDLDeviceDetailVo tdlDeviceDetailVo=new TDLDeviceDetailVo();
                tdlDeviceDetailVo.setGatewaySN(list.get(a).getSN());
                String sqls ="SELECT \"b\", \"T\", \"h\" FROM \"tdl_policy\".\""+list.get(a).getDevice() +"\" WHERE (\"tdl\" = '"+tdls.get(b)+"') and time>="+startTime*1000000l+" ORDER BY time DESC LIMIT 1";
                QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
                if (queryResults.getResults().get(0).getSeries() != null) {
                    tdlDeviceDetailVo.setTime(parseTime(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                    tdlDeviceDetailVo.setPressure(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                    tdlDeviceDetailVo.setTemperature(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    tdlDeviceDetailVo.setHumidity(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(3).toString());
                }else{
                    tdlDeviceDetailVo.setTime("NA");
                    tdlDeviceDetailVo.setPressure("NA");
                    tdlDeviceDetailVo.setTemperature("NA");
                    tdlDeviceDetailVo.setHumidity("NA");
                }
                tdlDeviceDetailVo.setTDLSN(tdls.get(b).replace("TDL-","").trim());
                tdlDeviceDetailVos.add(tdlDeviceDetailVo);
            }
            gatewayOnLineVo.setTdlDeviceDetailVos(tdlDeviceDetailVos);
            gatewayOnLineVos.add(gatewayOnLineVo);
        }
        warehouseDetailVo.setGatewayOnLineVos(gatewayOnLineVos);
        return warehouseDetailVo;
    }



    //根据仓库获取用户
    @ApiOperation(value="获取报警联系人",notes = "获取报警联系人，传值:仓库编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getAlarmContacts")
    public Object getAlarmContacts(@RequestBody String warehouseNo ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(warehouseNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        if(warehouseService.getWarehouseNo(warehouseNo,user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        return   warehouseService.getAlarmContacts(warehouseNo);
    }



    //获取某个仓库的网关
    @ApiOperation(value="添加网关",notes = "添加网关，传值:" +
            "warehouseNo:仓库编号" +
            "SN:网关编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getGateway")
    public Object getGateway(@RequestBody String warehouseNo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(warehouseNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        if(warehouseService.getWarehouseNo(warehouseNo,user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        //获取网关编号
        return warehouseService.getGateway(warehouseNo);
    }



    //添加仓库信息
    @ApiOperation(value = "添加仓库信息",notes = "添加仓库信息,传值说明：" +
            "   img:图片地址" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   city:市," +
            "   addressDetail:具体地址," +
            "   regionName:区域名," +
            "   property:属性," +
            "   type:类型," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addWarehouse",method = RequestMethod.POST)
    public Object addWarehouse(@RequestBody AddWarehouseVo addWarehouseVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        String warehouseNo = getWarehouseNo(user.getCompanyNo());
        if(StringUtils.isEmpty(addWarehouseVo.getImg())){
            addWarehouseVo.setImg("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/pic/wareHouseImage/1527562358242TM.jpg");
        }
        if (StringUtils.isEmpty(addWarehouseVo.getWarehouseName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"country"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"city"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getRegionName())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lng"));
            return gson.toJson(fb);
        }
        if(!addWarehouseVo.getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLat"));
            return gson.toJson(fb);
        }
        if(!addWarehouseVo.getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getProperty())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"property"));
            return gson.toJson(fb);
        }
        if (!i18n.getMessage(request,"warehouseProperty1").equals(addWarehouseVo.getProperty())
                &&!i18n.getMessage(request,"warehouseProperty2").equals(addWarehouseVo.getProperty())
                && !i18n.getMessage(request,"warehouseProperty3").equals(addWarehouseVo.getProperty())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongProperty"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseType"));
            return gson.toJson(fb);
        }
        if (!addWarehouseVo.getType().equals(i18n.getMessage(request,"warehouseType1"))
                &&!addWarehouseVo.getType().equals(i18n.getMessage(request,"warehouseType2"))){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongWarehouseType"));
            return gson.toJson(fb);
        }
//        if (StringUtils.isEmpty(addWarehouseVo.getHardwareLevel())){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"hardwareLevel"));
//            return gson.toJson(fb);
//        }
//        if (!addWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel1"))
//                &&!addWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel2"))
//                && !addWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel3"))
//                &&!addWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel4"))){
//            fb.setCode(0);
//            fb.setMessage(i18n.getMessage(request,"wrongHardwareLevel"));
//            return gson.toJson(fb);
//        }
        if (addWarehouseVo.getAreaIndoor()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaIndoor"));
            return gson.toJson(fb);
        }
        if (addWarehouseVo.getAreaRack()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaRack"));
            return gson.toJson(fb);
        }
        if (addWarehouseVo.getAreaOutdoor()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaOutdoor"));
            return gson.toJson(fb);
        }
        if (addWarehouseVo.getAreaPlanIndoor() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaPlanIndoor"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getOffice())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"office"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseVo.getTimeZone())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"timeZone"));
            return gson.toJson(fb);
        }
        if (warehouseService.getByWarehouseName(addWarehouseVo.getWarehouseName(),user.getCompanyNo())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeateWarehouseName"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",warehouseNo);
        map.put("warehouseName",addWarehouseVo.getWarehouseName());
        map.put("country",addWarehouseVo.getCountry());
        map.put("city",addWarehouseVo.getCity());
        map.put("addressDetail",addWarehouseVo.getAddressDetail());
        map.put("lng",addWarehouseVo.getLng());
        map.put("lat",addWarehouseVo.getLat());
        map.put("regionName",addWarehouseVo.getRegionName());
        map.put("property",addWarehouseVo.getProperty());
        map.put("type",addWarehouseVo.getType());
        map.put("hardwareLevel",addWarehouseVo.getHardwareLevel());
        map.put("areaIndoor",addWarehouseVo.getAreaIndoor());
        map.put("areaRack",addWarehouseVo.getAreaRack());
        map.put("areaOutdoor",addWarehouseVo.getAreaOutdoor());
        map.put("areaPlanIndoor",addWarehouseVo.getAreaPlanIndoor());
        map.put("office",addWarehouseVo.getOffice());
        map.put("timeZone",addWarehouseVo.getTimeZone());
        map.put("warehouseInfo",addWarehouseVo.getWarehouseInfo() == null ? "" :addWarehouseVo.getWarehouseInfo());
        map.put("companyNo",user.getCompanyNo());
        map.put("createTime",System.currentTimeMillis());
        map.put("updateTime",System.currentTimeMillis());
        map.put("img",addWarehouseVo.getImg());
        map.put("remark",addWarehouseVo.getRemark()== null ? "" :addWarehouseVo.getRemark());
        warehouseService.addWarehouse(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("添加仓库成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addWarehouseSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addWarehouseFailure"));
        }
        return gson.toJson(fb);

    }



    //根据仓库编号修改仓库信息
    @ApiOperation(value = "根据仓库编号修改仓库信息",notes = "根据仓库编号修改仓库信息,updatTime不用传，传值说明：" +
            "   warehouseName:仓库名," +
            "   country:国家," +
            "   city:市," +
            "   addressDetail:具体地址," +
            "   regionName:区域名," +
            "   property:属性," +
            "   type:类型," +
            "   hardwareLevel:硬件等级," +
            "   areaIndoor:室内平面面积," +
            "   areaRack:货架面积," +
            "   areaOutdoor:室外面积," +
            "   areaPlanIndoor:实际室内总面积," +
            "   office:办事处," +
            "   timeZone:时区," +
            "   warehouseInfo:基础描述," +
            "   remark:备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateWarehouse",method = RequestMethod.POST)
    public Object updateWarehouse(@RequestBody UpdateWarehouseVo updateWarehouseVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(updateWarehouseVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(updateWarehouseVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getWarehouseName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getCountry())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"country"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getCity())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"city"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getAddressDetail())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addressDetail"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getRegionName())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"regionName"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getLat())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lat"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getLng())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"lng"));
            return gson.toJson(fb);
        }
        if(!updateWarehouseVo.getLat().matches("^-?(([1-9])((\\.\\d{1,6})?)|([1-8]\\d)((\\.\\d{1,6})?)|90)$")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLat"));
            return gson.toJson(fb);
        }
        if(!updateWarehouseVo.getLng().matches("^-?(([1-9]\\d?)((\\.\\d{1,6})?)|(1[0-7]\\d)((\\.\\d{1,6})?)|180)$")) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongLng"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getProperty())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"property"));
            return gson.toJson(fb);
        }

        if (!updateWarehouseVo.getProperty().equals(i18n.getMessage(request,"warehouseProperty1"))
                &&!updateWarehouseVo.getProperty().equals(i18n.getMessage(request,"warehouseProperty2"))
                && !updateWarehouseVo.getProperty().equals(i18n.getMessage(request,"warehouseProperty3"))){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongProperty"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseType"));
            return gson.toJson(fb);
        }
        if (!i18n.getMessage(request,"warehouseType1").equals(updateWarehouseVo.getType())
                &&!i18n.getMessage(request,"warehouseType2").equals(updateWarehouseVo.getType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongWarehouseType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getHardwareLevel())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"hardwareLevel"));
            return gson.toJson(fb);
        }
        if (!updateWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel1"))
                &&!updateWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel2"))
                && !updateWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel3"))
                &&!updateWarehouseVo.getHardwareLevel().equals(i18n.getMessage(request,"warehouseHardwareLevel4"))){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongHardwareLevel"));
            return gson.toJson(fb);
        }
        if (updateWarehouseVo.getAreaIndoor() == null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaIndoor"));
            return gson.toJson(fb);
        }
        if (updateWarehouseVo.getAreaRack()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaRack"));
            return gson.toJson(fb);
        }
        if (updateWarehouseVo.getAreaOutdoor()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaOutdoor"));
            return gson.toJson(fb);
        }
        if (updateWarehouseVo.getAreaPlanIndoor()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"areaPlanIndoor"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getOffice())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"office"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateWarehouseVo.getTimeZone())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"timeZone"));
            return gson.toJson(fb);
        }
        ResultWarehouseVo resultWarehouseVo=warehouseService.getByWarehouseName(updateWarehouseVo.getWarehouseName(),user.getCompanyNo());
        if (resultWarehouseVo!=null&&!resultWarehouseVo.getWarehouseNo().equals(updateWarehouseVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"repeateWarehouseName"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",updateWarehouseVo.getWarehouseNo());
        map.put("warehouseName",updateWarehouseVo.getWarehouseName());
        map.put("country",updateWarehouseVo.getCountry());
        map.put("city",updateWarehouseVo.getCity());
        map.put("addressDetail",updateWarehouseVo.getAddressDetail());
        map.put("lng",updateWarehouseVo.getLng());
        map.put("lat",updateWarehouseVo.getLat());
        map.put("regionName",updateWarehouseVo.getRegionName());
        map.put("property",updateWarehouseVo.getProperty());
        map.put("type",updateWarehouseVo.getType());
        map.put("hardwareLevel",updateWarehouseVo.getHardwareLevel());
        map.put("areaIndoor",updateWarehouseVo.getAreaIndoor());
        map.put("areaRack",updateWarehouseVo.getAreaRack());
        map.put("areaOutdoor",updateWarehouseVo.getAreaOutdoor());
        map.put("areaPlanIndoor",updateWarehouseVo.getAreaPlanIndoor());
        map.put("office",updateWarehouseVo.getOffice());
        map.put("timeZone",updateWarehouseVo.getTimeZone());
        map.put("warehouseInfo",updateWarehouseVo.getWarehouseInfo());
        map.put("companyNo",user.getCompanyNo());
        map.put("updateTime",System.currentTimeMillis());
        map.put("remark",updateWarehouseVo.getRemark());
        warehouseService.updateWarehouse(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("修改仓库成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedWarehouseSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedWarehouseFailure"));
        }
        return gson.toJson(fb);
    }



    //删除仓库信息
    @ApiOperation(value = "删除仓库信息",notes = "删除仓库信息,只用传仓库编号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delWarehouse",method = RequestMethod.POST)
    public Object delWarehouse(@RequestBody String warehouseNo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(warehouseNo)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(warehouseNo);
        warehouseNo=(String)jsonObject.get("warehouseNo");
        if(warehouseService.getWarehouseNo(warehouseNo,user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",warehouseNo);
        map.put("companyNo",user.getCompanyNo());
        map.put("endTime",System.currentTimeMillis());
        warehouseService.delWarehouse(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("删除仓库成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deleteWarehouseSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deleteWarehouseFailure"));
        }
        return gson.toJson(fb);
    }



    @ApiOperation(value="添加网关",notes = "添加网关，传值:" +
            "warehouseNo:仓库编号" +
            "SN:网关编号" +
            "type:网关类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/addGateway")
    public Object addGateway(@RequestBody AddWarehouseGatewayVo addWarehouseGatewayVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addWarehouseGatewayVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(addWarehouseGatewayVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseGatewayVo.getgSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if( gatewayService.getBySN(addWarehouseGatewayVo.getgSN(),user.getCompanyNo())==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"existGateway"));
            return gson.toJson(fb);
        }
        String type = gatewayService.getBySN(addWarehouseGatewayVo.getgSN(),user.getCompanyNo());
        //判断当前网关是否已经被使用
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",addWarehouseGatewayVo.getWarehouseNo());
        map.put("gSN",addWarehouseGatewayVo.getgSN());
        map.put("gType",type);
        map.put("startTime",System.currentTimeMillis());
        warehouseService.addGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("添加网关成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"bindingGatewaySuccess"));
        }else{
            fb.setCode(0);
//            fb.setMessage(msg.get("msg").toString());
            fb.setMessage(i18n.getMessage(request,"bindingGatewayFailure"));
        }
        return gson.toJson(fb);
    }



    //绑定报警联系人
    @ApiOperation(value="绑定报警联系人",notes = "绑定报警联系人，传值:" +
            "warehouseNo:仓库编号" +
            "userNameList:用户名的集合")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/addUser")
    public Object addUser(@RequestBody WarehouseUserVo warehouseUserVo, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(warehouseUserVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(warehouseUserVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        if (warehouseUserVo.getUserNameList().size() == 0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"userName"));
            return gson.toJson(fb);
        }
        String userNameVar = "";
        for(int i = 0,length=warehouseUserVo.getUserNameList().size();i<length;i++){
            if(userService.getByUserName(warehouseUserVo.getUserNameList().get(i).getUserName()) == null){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"NonexistentUser"));
                return gson.toJson(fb);
            }
            userNameVar = userNameVar +warehouseUserVo.getUserNameList().get(i).getUserName() +"&";
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",warehouseUserVo.getWarehouseNo());
        map.put("number",warehouseUserVo.getUserNameList().size());
        map.put("userNameVar",userNameVar);
        warehouseService.addUser(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("添加紧急联系人成功")){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addAlarmContactsSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addAlarmContactsFailure"));
        }
        return  gson.toJson(fb);
    }



    //根据仓库编号获取仓库实时数据
    @ApiOperation(value = "根据仓库编号获取仓库信息",notes = "根据仓库编号获取仓库信息,传值说明：" +
            "   warehouseNo:仓库编号，" +
            "返回值：" +
            "   [" +
            "       {" +
            "          gSN: 网关编号," +
            "          type: 网关类型," +
            "          warehouseDataVos: [" +
            "               {" +
            "                   warehouseTDLDetailVos: [" +
            "                       description: 描述," +
            "                       max: 最大值," +
            "                       min: 最小值" +
            "                       timeAndValuesVos: {" +
            "                           time：时间，" +
            "                           value：值" +
            "                       }" +
            "                   ]," +
            "                   tdlsn: TDL-201802271711" +
            "               }" +
            "           ]" +
            "       }" +
            "   ]")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getWarehouseData")
    public Object getWarehouseData(@RequestBody WarehouseVo warehouseVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if(warehouseService.getWarehouseNo(warehouseVo.getWarehouseNo(),user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        List<WarehouseGatewayDateVo> warehouseGatewayDateVoList=new ArrayList<>();
        List<GatewayOnLineVo> gatewayOnLineVos=gatewayService.getByWarehouseNo(warehouseVo.getWarehouseNo());
        for (int a=0;a<gatewayOnLineVos.size();a++){
            if(gatewayOnLineVos.get(a) == null){
                continue;
            }
            WarehouseGatewayDateVo warehouseGatewayDateVo=new WarehouseGatewayDateVo();
            warehouseGatewayDateVo.setgSN(gatewayOnLineVos.get(a).getgSN());
            warehouseGatewayDateVo.setType(gatewayOnLineVos.get(a).getType());
            String device = gatewayOnLineVos.get(a).getType() + "_" + gatewayOnLineVos.get(a).getgSN();
            String sql ="SELECT \"battery\" FROM \"tdl_policy\".\""+device +"\" where  time >="+ gatewayOnLineVos.get(a).getStartTime()*1000000l+" ORDER BY time DESC LIMIT 1";
            QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
            if (queryResult.getResults().get(0).getSeries() != null) {
                if(Double.valueOf(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString())>0){
                    warehouseGatewayDateVo.setBattery(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1)==null ? "NA" :queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                }else{
                    warehouseGatewayDateVo.setBattery("NA");
                }
            }else {
                warehouseGatewayDateVo.setBattery("NA");
            }
            Object data = infoRedisService.getHash("DeviceOnline", device );
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 1){
                    warehouseGatewayDateVo.setState(7);
                }else {
                    warehouseGatewayDateVo.setState(6);
                }
            }else{
                warehouseGatewayDateVo.setState(6);
            }
            List<String> tdls=tdlDeviceService.getByGatewaySN(gatewayOnLineVos.get(a).getgSN(),gatewayOnLineVos.get(a).getType());
            List<WarehouseDataVo> warehouseDataVos=new ArrayList<>();
            for (int b=0;b<tdls.size();b++){
                List<WarehouseTDLDetailVo> warehouseTDLDetailVos=new ArrayList<>();
                WarehouseDataVo warehouseDataVo=new WarehouseDataVo();
                warehouseDataVo.setTDLSN(tdls.get(b).replace("TDL-","").trim());
                String sqls ="SELECT \"tdl\",\"b\",\"T\",\"h\" FROM \"tdl_policy\".\""+device+"\" WHERE \"tdl\" = '"+tdls.get(b)+"'  AND  time >="+  gatewayOnLineVos.get(a).getStartTime()*1000000l+"  ORDER BY time  desc  limit 1 ";
                QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
                ResultAlarmVo resultAlarmVo=alarmService.getAlarmByWarehouseNo(warehouseVo.getWarehouseNo(),gatewayOnLineVos.get(a).getgSN(),tdls.get(b).replace("TDL-","").trim());
                if (queryResults.getResults().get(0).getSeries() != null) {
                    warehouseDataVo.setTime(UTCToCST(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString(),warehouseVo.getOffset()==null ? "+8" : warehouseVo.getOffset()));
                    warehouseDataVo.setBatteryVoltage(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    warehouseDataVo.setTemperature(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(3).toString());
                    warehouseDataVo.setHumidity(queryResults.getResults().get(0).getSeries().get(0).getValues().get(0).get(4).toString());
                }else{
                    warehouseDataVo.setTime("NA");
                    warehouseDataVo.setBatteryVoltage("NA");
                    warehouseDataVo.setTemperature("NA");
                    warehouseDataVo.setHumidity("NA");
                }
                String sql3 ="SELECT \"tdl\",\"T\",\"h\" FROM \"tdl_policy\".\""+device+"\" WHERE \"tdl\" = '"+tdls.get(b)+"'  AND  time >="+  gatewayOnLineVos.get(a).getStartTime()*1000000l+"  ORDER BY time ";
                QueryResult queryResult3 = influxDBTemplate.query(new Query(sql3, database));
                if(queryResult3.getResults().get(0).getSeries() != null){

                    warehouseTDLDetailVos = getWarehouseData(queryResult3,warehouseVo.getOffset()==null ? "+8" : warehouseVo.getOffset(),resultAlarmVo,warehouseDataVo);
                    //解析数据
//                    WarehouseTDLDetailVo warehouseTDLDetailVo=new WarehouseTDLDetailVo();
//                    warehouseTimeAndDataVos = getData(queryResult3,warehouseVo.getOffset()==null ? "+8" : warehouseVo.getOffset());
//                    warehouseTDLDetailVo.setDescription("temperature");
//                    warehouseTDLDetailVo.setTimeAndValuesVos(warehouseTimeAndDataVos);
//                    warehouseTDLDetailVo.setMax(resultAlarmVo.getTemMax());
//                    warehouseTDLDetailVo.setMin(resultAlarmVo.getTemMin());
//                    warehouseTDLDetailVos.add(warehouseTDLDetailVo);
//
//                    WarehouseTDLDetailVo warehouseTDLDetailVo1=new WarehouseTDLDetailVo();
//                    warehouseTimeAndDataVos1 = getDatas(queryResult3,warehouseVo.getOffset()==null ? "+8" : warehouseVo.getOffset());
//                    warehouseTDLDetailVo1.setDescription("humidity");
//                    warehouseTDLDetailVo1.setTimeAndValuesVos(warehouseTimeAndDataVos1);
//                    warehouseTDLDetailVo1.setMax(resultAlarmVo.getHumidityMax());
//                    warehouseTDLDetailVo1.setMin(resultAlarmVo.getHumidityMin());
//                    warehouseTDLDetailVos.add(warehouseTDLDetailVo1);
                }
                warehouseDataVo.setWarehouseTDLDetailVos(warehouseTDLDetailVos);
                warehouseDataVos.add(warehouseDataVo);
            }
            warehouseGatewayDateVo.setWarehouseDataVos(warehouseDataVos);
            warehouseGatewayDateVoList.add(warehouseGatewayDateVo);
        }
        return warehouseGatewayDateVoList;
    }



    //货物仓库某一tdl的温湿度
    @ApiOperation(value="货物仓库某一tdl的温湿度",notes = "货物仓库某一tdl的温湿度，传值:" +
            "   warehouseNo:仓库编号，" +
            "   sn:网关编号，" +
            "   tdlSN：TDL编号，" +
            "返回值说明：" +
            "   {" +
            "       gSN: 网关编号," +
            "       tdl: TDL编号," +
            "       warehouseName: 仓库名," +
            "       temp: 温度," +
            "       humidity: 湿度," +
            "       historyDataList: [" +
            "           {" +
            "               time:时间，" +
            "               temp:温度，" +
            "               humidity:湿度" +
            "           }" +
            "       ]," +
            "       graphicDataList: [" +
            "           {" +
            "               descption:描述，" +
            "               max:最大值，" +
            "               min:最小值" +
            "           }" +
            "       ]," +
            "   }")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getTdlData")
    public Object getTdlData(@RequestBody SearchTdlVo searchTdlVo ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(searchTdlVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(searchTdlVo.getWarehouseNo(), user.getCompanyNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        //判断网关和tdl是否存在
        if (StringUtils.isEmpty(searchTdlVo.getSN())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"gatewaySN"));
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(searchTdlVo.getTDLSN())) {
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return gson.toJson(fb);
        }
        //获取tdl的信息
        DeviceMessageVo deviceMessageVo =   warehouseService.getDeviceMessage(searchTdlVo);
        TdlDataVo tdlDataVo = new TdlDataVo();
        if(deviceMessageVo == null){
            return  tdlDataVo;
        }
        tdlDataVo.setgSN(deviceMessageVo.getSN());
        tdlDataVo.setTdl(deviceMessageVo.getTDLName());
        tdlDataVo.setWarehouseName(deviceMessageVo.getWarehouseName());
        List<GraphicDataVo>  graphicDataList = new ArrayList<>();
        List<TdlDataVo.HistoryDataVo> historyDataList = new ArrayList<>();
        //获取TDL数据
        String sql = "SELECT \"T\",\"h\" FROM \"tdl_policy\".\""+deviceMessageVo.getType()+"_"+deviceMessageVo.getSN()+"\"  where  \"tdl\" = \'TDL-"+searchTdlVo.getTDLSN()+"\'  order by time ";
        QueryResult queryResult = influxDBTemplate.query(new Query(sql,database));
        if(queryResult.getResults().get(0).getSeries()== null){
            tdlDataVo.setGraphicDataList(graphicDataList);
            tdlDataVo.setHistoryDataList(historyDataList);
        }
        tdlDataVo =   parseData(queryResult,deviceMessageVo.getHumidityMax(),deviceMessageVo.getHumidityMin(),deviceMessageVo.getTemMax(),deviceMessageVo.getTemMin());
        return  tdlDataVo;
    }



    //模糊查询所有的仓库名
    @ApiOperation(value="模糊查询所有的仓库名",notes = "模糊查询所有的仓库名，传值:" +
            "warehouseName:仓库名称")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/getWarehouseName")
    public Object getWarehouseName(@RequestBody String warehouseName ,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(warehouseName)){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseName"));
            return gson.toJson(fb);
        }
        JSONObject jsonObject=JSON.parseObject(warehouseName);
        warehouseName=(String) jsonObject.get("warehouseName");
        return warehouseService.getWarehouseName(warehouseName,user.getCompanyNo());
    }



    //生成仓库编号
    public String getWarehouseNo(String companyNo){
        String warehouseNo = "WH" +System.currentTimeMillis() ;
        warehouseNo = warehouseNo +((int)(Math.random()*(9999-1000+1))+1000);
        if (warehouseService.getWarehouseNo(warehouseNo,companyNo)>0){
            while (warehouseService.getWarehouseNo(warehouseNo,companyNo)>0){
                warehouseNo=warehouseNo+((int)(Math.random()*(9999-1000+1))+1000);
            }
        }
        return warehouseNo;
    }

    public List<WarehouseTDLDetailVo> getWarehouseData(QueryResult queryResult,String offset,ResultAlarmVo resultAlarmVo,WarehouseDataVo warehouseDataVo){
        List<WarehouseTDLDetailVo> warehouseTDLDetailVoList=new ArrayList<>();
        List<List<Object>> values = getValues(queryResult);
        List<WarehouseTimeAndDataVo> historyDataList = new ArrayList<>();
        List<WarehouseTimeAndDataVo> historyDataLists = new ArrayList<>();
        for(List<Object> value:values){
            String time = UTCToCST(value.get(0).toString(),offset);
            WarehouseTimeAndDataVo warehouseTimeAndDataVo = new WarehouseTimeAndDataVo();
            warehouseTimeAndDataVo.setTime(time);
            warehouseTimeAndDataVo.setValue(value.get(3)==null  ? "" :value.get(3).toString());
            historyDataList.add(warehouseTimeAndDataVo);

            WarehouseTimeAndDataVo warehouseTimeAndDataVos = new WarehouseTimeAndDataVo();
            warehouseTimeAndDataVos.setTime(time);
            warehouseTimeAndDataVos.setValue(value.get(2)==null  ? "" :value.get(2).toString());
            historyDataLists.add(warehouseTimeAndDataVos);
        }
       // if(!warehouseDataVo.getTemperature().equals("NA")){
         //   historyDataLists.add(new WarehouseTimeAndDataVo(warehouseDataVo.getTime(),warehouseDataVo.getTemperature()));
        //    historyDataList.add(new WarehouseTimeAndDataVo(warehouseDataVo.getTime(),warehouseDataVo.getHumidity()));
       // }
        WarehouseTDLDetailVo warehouseTDLDetailVo=new WarehouseTDLDetailVo();
        warehouseTDLDetailVo.setDescription("temperature");
        warehouseTDLDetailVo.setTimeAndValuesVos(historyDataLists);
        warehouseTDLDetailVo.setMax(resultAlarmVo.getTemMax());
        warehouseTDLDetailVo.setMin(resultAlarmVo.getTemMin());
        warehouseTDLDetailVoList.add(warehouseTDLDetailVo);

        WarehouseTDLDetailVo warehouseTDLDetailVos=new WarehouseTDLDetailVo();
        warehouseTDLDetailVos.setDescription("humidity");
        warehouseTDLDetailVos.setTimeAndValuesVos(historyDataList);
        warehouseTDLDetailVos.setMax(resultAlarmVo.getHumidityMax());
        warehouseTDLDetailVos.setMin(resultAlarmVo.getHumidityMin());
        warehouseTDLDetailVoList.add(warehouseTDLDetailVos);
        return warehouseTDLDetailVoList;
    }


    public List<WarehouseTimeAndDataVo> getData(QueryResult queryResult,String offset){
        List<List<Object>> values = getValues(queryResult);
        List<WarehouseTimeAndDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            WarehouseTimeAndDataVo warehouseTimeAndDataVo = new WarehouseTimeAndDataVo();
            warehouseTimeAndDataVo.setTime(UTCToCST(value.get(0).toString(),offset));
            warehouseTimeAndDataVo.setValue(value.get(3)==null  ? "" :value.get(3).toString());
            historyDataList.add(warehouseTimeAndDataVo);
        }
        return historyDataList;
    }



    public List<WarehouseTimeAndDataVo> getDatas(QueryResult queryResult,String offset){
        List<List<Object>> values = getValues(queryResult);
        List<WarehouseTimeAndDataVo> historyDataList = new ArrayList<>();
        for(List<Object> value:values){
            WarehouseTimeAndDataVo warehouseTimeAndDataVo = new WarehouseTimeAndDataVo();
            warehouseTimeAndDataVo.setTime(UTCToCST(value.get(0).toString(),offset));
            warehouseTimeAndDataVo.setValue(value.get(4)==null ? "" :value.get(4).toString());
            historyDataList.add(warehouseTimeAndDataVo);
        }
        return historyDataList;
    }



    //解析数据格式
    public List<List<Object>>  getValues( QueryResult  queryResult){
        List<List<Object>> values = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        values= series.getValues();
                    }
                }
            }
        }
        return values;
    }



    //解析时间
    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }



    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }

    public static  String UTCToCST(String UTCStr,String  offset) {
        Date date = null;
        try {
            if (UTCStr.length() != 20) {
                date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(UTCStr);
            } else {
                date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(UTCStr);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) +Integer.valueOf(offset));
        //calendar.getTime() 返回的是Date类型，也可以使用calendar.getTimeInMillis()获取时间戳
        //format的格式可以任意
        return String.valueOf(calendar.getTimeInMillis());
    }



    public SimpleWarehouseVo  dataData(QueryResult queryResult){
        SimpleWarehouseVo simpleWarehouseVo = new SimpleWarehouseVo();
        List<List<Object>> value = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        value.add(series.getValues().get(0));
                    }else{
                        continue;
                    }
                }
            }else{
                continue;
            }
        }
        List<Double> tempList = new ArrayList<>();
        List<Double> humList = new ArrayList<>();
        for(int i = 0,length = value.size();i<length;i++){
            DecimalFormat df = new DecimalFormat(".0");
            tempList.add(value.get(i).get(1)==null ? 0 :Double.valueOf(df.format(Double.valueOf(value.get(i).get(1).toString()))));
            humList.add(value.get(i).get(2)==null ?  0 :Double.valueOf(df.format(Double.valueOf(value.get(i).get(2).toString()))));
        }
        //设置数据
        simpleWarehouseVo.setTemp(getValue(tempList));
        simpleWarehouseVo.setHumidity(getValue(humList));
        return  simpleWarehouseVo;
    }



    public String  getValue(List<Double> data){
        String s = "";
        Collections.sort(data);
        if(data.size() == 1){
            if(data.get(0) == 0){
                return "~";
            }
            return data.get(0) +"~" +data.get(0);
        }else{
            if(data.get(0)!=0 && data.get(data.size()-1)!=0){
                return data.get(0) +"~" +data.get(data.size()-1);
            }else{
                for(int i=0;i<data.size();i++){
                    if(data.get(0)==0){
                        continue;
                    }else{
                        s=data.get(i).toString();
                    }
                }
                if(StringUtils.isEmpty(s)){
                    return "~";
                }
                return s+"~"+data.get(data.size()-1);
            }
        }
    }



    public  TdlDataVo parseData(QueryResult queryResult,String humidityMax,String humidityMin,String tempMax,String tempMin){
        TdlDataVo tdlDataVo = new TdlDataVo();
        List<List<Object>> values = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        values = series.getValues();
                    }else{
                        continue;
                    }
                }
            }else{
                continue;
            }
        }
        List<GraphicDataVo>  graphicDataList = new ArrayList<>();
        GraphicDataVo temp = new GraphicDataVo();
        temp.setDescption("temp");
        temp.setMax(tempMax);
        temp.setMin(tempMin);
        List<HistoryData> tempData = new ArrayList<>();
        GraphicDataVo hum = new GraphicDataVo();
        hum.setDescption("humidity");
        hum.setMin(humidityMin);
        hum.setMax(humidityMax);
        List<HistoryData> humpData = new ArrayList<>();
        List<TdlDataVo.HistoryDataVo> historyDataList = new ArrayList<>();
        for (int i =0,length = values.size();i<length;i++){
            TdlDataVo.HistoryDataVo historyDataVo = new TdlDataVo.HistoryDataVo();
            historyDataVo.setHumidity(values.get(i).get(2)==null ?"" :values.get(i).get(2).toString() );
            historyDataVo.setTemp(values.get(i).get(1)==null ?"" :values.get(i).get(1).toString() );
            String time =UTCToCST(values.get(i).get(0).toString());
            historyDataVo.setTime(Long.valueOf(time));
            historyDataList.add(historyDataVo);
            HistoryData tem = new HistoryData();
            tem.setTime(time);
            tem.setValue(values.get(i).get(1)==null ?"" :values.get(i).get(1).toString());
            tempData.add(tem);
            HistoryData hump = new HistoryData();
            hump.setTime(time);
            hump.setValue(values.get(i).get(2)==null ?"" :values.get(i).get(2).toString());
            humpData.add(hump);
            if( i == length-1){
                tdlDataVo.setHumidity(values.get(i).get(2)==null ?"" :values.get(i).get(2).toString());
                tdlDataVo.setTemp(values.get(i).get(1)==null ?"" :values.get(i).get(1).toString());
            }
        }
        temp.setHistoryDataList(tempData);
        hum.setHistoryDataList(humpData);
        graphicDataList.add(temp);
        graphicDataList.add(hum);
        tdlDataVo.setHistoryDataList(historyDataList);
        tdlDataVo.setGraphicDataList(graphicDataList);
        return  tdlDataVo;
    }



    public  String UTCToCST(String UTCStr) {
        Date date =null ;
        try {
            if (UTCStr.length() != 20) {
                date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(UTCStr);
            } else {
                date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(UTCStr);
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR, calendar.get(Calendar.HOUR) + 8);
        //calendar.getTime() 返回的是Date类型，也可以使用calendar.getTimeInMillis()获取时间戳
        //format的格式可以任意
        return String.valueOf(calendar.getTimeInMillis());
    }


}
